VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CTokens"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'*****************************************************************************
'
' Classname     : CSLIDATokens (Code)
'
' Description   : Reading tokens from mailslot file
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'
'  2000 Sirona Dental Systems GmbH - All Rights Reserved
'-----------------------------------------------------------------------------
' Status        : Design
'
' Security      : -
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -      Modification
' 28.01.00 -      Design
'
'*****************************************************************************
Option Explicit
Option Base 1

'*****************************************************************************
'Private definitions and constants
'*****************************************************************************
Private m_fH                As Long         'internal file handle
Private m_ColSlida          As Collection   'all read tokens
Private m_Err               As Long         'internal error number
Private m_Convert           As CStrConv     'String OEM - ANSI converter class

'*****************************************************************************
'Private data types
'*****************************************************************************

'*****************************************************************************
' Global variables and constants
'*****************************************************************************

Public Event InvalidTokenFound()

'*****************************************************************************
' Implementation
'*****************************************************************************

'*****************************************************************************
'
' Function name : Class_Initialize
'
' Description   : Init for SLIDA Collection and converter Class
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : -
'
' Extern data   : Name                          Access (r/w/&)
'                 m_ColSlida                    w
'                 m_Convert                     w
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Private Sub Class_Initialize()
   Set m_ColSlida = New Collection
   Set m_Convert = New CStrConv
End Sub

'*****************************************************************************
'
' Function name : Class_Terminate
'
' Description   : destroy SLIDA Collection und converter Class
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : -
'
' Extern data   : Name                          Access (r/w/&)
'                 m_ColSlida                    w
'                 m_Convert                     w
'
' Remarks       : open mailslot file will be closed as well
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Private Sub Class_Terminate()
   Set m_ColSlida = Nothing
   Set m_Convert = Nothing

   CloseMailslot
End Sub

'*****************************************************************************
'
' Function name : Property Get Item
'
' Description   : used for referenzing an element of the collection
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : vntIndexKey As Variant        / Index or Key
'
' Returns       : Reference to indexed element of CToken class
'
' Extern data   : Name                          Access (r/w/&)
'                 m_ColSlida                    r
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Property Get Item(vntIndexKey As Variant) As CToken
   Set Item = m_ColSlida(vntIndexKey)
End Property

'*****************************************************************************
'
' Function name : Property Get Count
'
' Description   : Returns the count of token elements
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : the count of token elements in m_ColSilda
'
' Extern data   : Name                          Access (r/w/&)
'                 m_ColSlida                    r
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Property Get Count() As Long
   Count = m_ColSlida.Count
End Property

'*****************************************************************************
'
' Function name : Property Get NewEnum
'
' Description   : Allows enumeration via For ... Each
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : an element for iteration
'
' Extern data   : Name                          Access (r/w/&)
'                 m_ColSlida                    r
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Property Get NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
   Set NewEnum = m_ColSlida.[_NewEnum]
End Property

'*****************************************************************************
'
' Function name : Sub Remove
'
' Description   : Removed an element of m_ColSlida
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : vntIndexKey As Variant / Index or Key of Collection
'
' Returns       : -
'
' Extern data   : Name                          Access (r/w/&)
'                 m_ColSlida                    w
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Sub Remove(vntIndexKey As Variant)
   m_ColSlida.Remove vntIndexKey
End Sub

'*****************************************************************************
'
' Function name : Clear
'
' Description   : removes all elements of collection m_ColSlida
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : -
'
' Extern data   : Name                          Access (r/w/&)
'                 m_ColSlida                    w
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Sub Clear()
   Dim Col     As CToken
  
   For Each Col In m_ColSlida
      Remove 1
   Next

End Sub

'*****************************************************************************
'
' Function name : OpenMailslot
'
' Description   : Opens mailslot exklusive
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : strFile As String
'
' Returns       : True on success, else False
'
' Extern data   : Name                          Access (r/w/&)
'                 m_fH                          w
'                 m_Err                         w
'
' Remarks       : File must exist!
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
' 19.07.99 -     Fehlerbehandlung
'
'*****************************************************************************
Public Function OpenMailslot(strFile As String) As Boolean
  m_fH = FreeFile
  
  If m_fH = 0 Then
    m_Err = -1
    Exit Function  'Error there are no more handles
  End If
  
  On Error Resume Next
  
  Open strFile For Binary Access Read Lock Read As #m_fH
  m_Err = Err                     'set internal error
  OpenMailslot = Err = 0          'return Fasle or True
End Function

'*****************************************************************************
'
' Function name : CloseMailslot
'
' Description   : Close the exklusivly opened mailslot file
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : -
'
' Extern data   : Name                          Access (r/w/&)
'                 m_fH                          &
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Function CloseMailslot()
  On Error Resume Next        'ignore close error if already closed
  Close #m_fH
End Function

'*****************************************************************************
'
' Function name : ReadMailslot
'
' Description   : Reading tokens from mailslot and add them to the collection
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : True on success, else False
'
' Extern data   : Name                          Access (r/w/&)
'                 m_Err                         w
'                 m_fH                          r
'                 m_ColSlida                    w
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Function ReadMailslot() As Boolean
  Dim strLine       As String     'read line
  Dim iLen          As Integer    'len of information
  Dim aBytData()    As Byte       'Array holding bytes read
  Dim lPos          As Long       'currend pointer of file position
  Dim cTok          As CToken     'one element of Token Class to be added to collection
  
  On Error GoTo ProcErr
  
  m_Err = 0         'reset error code
  
  If m_fH = 0 Then
    Debug.Assert False
    Exit Function  ' programmers error!
  End If
  
  Seek #m_fH, 1     'begin of file
  
  Do While Not EOF(m_fH)
   
    Get #m_fH, , iLen 'first get get the len (1 word), int should be fine (there is no data type word)
    
    If iLen = 0 Then Exit Do  'nothing in, so leave
  
    ReDim aBytData(1 To iLen - 2) '2 bytes less = drop CrLf
    Get #m_fH, , aBytData
    lPos = 2  ' data begins at pos = 2
          
    Select Case Chr(aBytData(1))    'which kind of token
      
      '-------------------------------------------------------------------------------
      Case "A"    'AutoSelectPatient / automatic patient/image selection
      '-------------------------------------------------------------------------------
        Set cTok = New CToken
        With cTok
          'put all data into single token instance
          .TokenID = "A"    'Token identifier
          
          'Expected data fields / Identification:
          
          '1. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '2. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '3. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
            
          '4. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
             
          'Expected data fields / Information:
          
          '1. Station name of the calling station
          .CallingStation = Byte2String(aBytData, lPos, lPos)
          
          '2. Date of call
          .OrderDate = Byte2String(aBytData, lPos, lPos)
                    
          '3. Time of call
          .OrderTime = Byte2String(aBytData, lPos, lPos)
          
          '4. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '5. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
          
          'Expected data fields / Identification:
        
          '1. Image number (optional), 0 for image selection
          .ImageNo = Byte2String(aBytData, lPos, lPos)
          
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With
        
      '-------------------------------------------------------------------------------
      Case "I"    'Information / Changing supplementary image data
      '-------------------------------------------------------------------------------
        Set cTok = New CToken
        With cTok
          'put all data into single token instance
          .TokenID = "I"  'Token identifier
          
          'Expected data fields / Identification:
          
          '1. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '2. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '3. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '4. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          '5. Image number (optional), 0 for image selection
          .ImageNo = Byte2String(aBytData, lPos, lPos)
     
          'Expected data fields / Information:
          
          '1. Image type (*)
          .ImageType = Byte2String(aBytData, lPos, lPos)
          
          '2. Date of image (*)
          .DateOfImage = Byte2String(aBytData, lPos, lPos)
          
          '3. Time of image (*)
          .TimeOfImage = Byte2String(aBytData, lPos, lPos)
          
          '4. Person in charge (*)
          .ResponsiblePerson = Byte2String(aBytData, lPos, lPos)
          
          '5. Reason/Notes (*)
          .Reason = Byte2String(aBytData, lPos, lPos)
          
          '6. Diagnosis (*)
          .Diagnosis = Byte2String(aBytData, lPos, lPos)
          
          '7. Radiation duration (*)
          .RadDuration = Byte2String(aBytData, lPos, lPos)
          
          '8. Tube voltage (*)
          .TupeVoltage = Byte2String(aBytData, lPos, lPos)
          
          '9. Tube current (*)
          .TupeCurrent = Byte2String(aBytData, lPos, lPos)
          
          '10. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '11. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
     
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With
      
      '-------------------------------------------------------------------------------
      Case "M"    'Mail / Image transfer
      '-------------------------------------------------------------------------------
        Set cTok = New CToken
        With cTok
          'put all data into single token instance
          .TokenID = "M"  'Token identifier
          
          'Expected data fields / Identification:
          
          '1. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '2. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '3. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '4. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          'Expected data fields / Information:
          
          '1. Image type
          .ImageType = Byte2String(aBytData, lPos, lPos)
          
          '2. Date of image
          .DateOfImage = Byte2String(aBytData, lPos, lPos)
          
          '3. Time of image
          .TimeOfImage = Byte2String(aBytData, lPos, lPos)
          
          '4. Person in charge
          .ResponsiblePerson = Byte2String(aBytData, lPos, lPos)
          
          '5. Image file name without path
          .ImageFilename = Byte2String(aBytData, lPos, lPos)
          
          '6. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '7. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
          
          'Expected data fields / Identification / Addition:
          
          '1. Image number
          .ImageNo = Byte2String(aBytData, lPos, lPos)
          
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With

      '-------------------------------------------------------------------------------
      Case "N"    'NewPatient / New registration of a patient
      '-------------------------------------------------------------------------------
        Set cTok = New CToken
        With cTok
          'put all data into single token instance
          .TokenID = "N"  'Token identifier
          

          'Expected data fields / Identification:
          '-
          
          'Expected data fields / Information:
          
          '1. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '2. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '3. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '4. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          '5. Sex
          .Sex = Byte2String(aBytData, lPos, lPos)
          
          '6. Permanent dentist
          .PermanentDentist = Byte2String(aBytData, lPos, lPos)
          
          '7. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '8. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
      
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With
      
      '-------------------------------------------------------------------------------
      Case "R"    'Ready Token
      '-------------------------------------------------------------------------------
        'put all data into single token instance
        Set cTok = New CToken
        With cTok
          .TokenID = "R"   'Token identifier
      
          '1. Order number
          .OrderNo = Byte2String(aBytData, lPos, lPos)
          
          '2. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '3. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '4. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '5. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          '6. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '7. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
          
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With
      
      '-------------------------------------------------------------------------------
      Case "S"    'SelectPatient / Preselection of patient
      '-------------------------------------------------------------------------------
        'put all data into single token instance
        Set cTok = New CToken
        With cTok
          .TokenID = "S"   'Token identifier
      

          'Expected data fields / Identification:
          
          '1. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '2. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '3. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '4. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          'Expected data fields / Information:
          
          '1. Station name of the calling station
          .CallingStation = Byte2String(aBytData, lPos, lPos)
          
          '2. Date of call
          .OrderDate = Byte2String(aBytData, lPos, lPos)
          
          '3. Time of call
          .OrderTime = Byte2String(aBytData, lPos, lPos)
          
          '4. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '5. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
      
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With
      
      '-------------------------------------------------------------------------------
      Case "T"    'Take / Producing an image
      '-------------------------------------------------------------------------------
        'put all data into single token instance
        Set cTok = New CToken
        With cTok
          .TokenID = "T"   'Token identifier
          
          'Expected data fields / Identification:
          
          '1. Order number
          .OrderNo = Byte2String(aBytData, lPos, lPos)
          
          '2. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '3. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '4. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '5. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          '6. Image number
          .ImageNo = Byte2String(aBytData, lPos, lPos)
     
          'Expected data fields / Information:
     
          '1. Sex
          .Sex = Byte2String(aBytData, lPos, lPos)
          
          '2. Pregnancy statement
          .pregnant = Byte2String(aBytData, lPos, lPos)
          
          '3. Image type
          .ImageType = Byte2String(aBytData, lPos, lPos)
          
          '4. Date of image
          .DateOfImage = Byte2String(aBytData, lPos, lPos)
          
          '5. Time of image
          .TimeOfImage = Byte2String(aBytData, lPos, lPos)
          
          '6. Person in charge
          .ResponsiblePerson = Byte2String(aBytData, lPos, lPos)
          
          '7. Reason/Notes
          .Reason = Byte2String(aBytData, lPos, lPos)
          
          '8. Diagnosis
          .Diagnosis = Byte2String(aBytData, lPos, lPos)
          
          '9. Radiation duration
          .RadDuration = Byte2String(aBytData, lPos, lPos)
          
          '10. Tube voltage
          .TupeVoltage = Byte2String(aBytData, lPos, lPos)
          
          '11. Tube current
          .TupeCurrent = Byte2String(aBytData, lPos, lPos)
          
          '12. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '13. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
               
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With
  
      '-------------------------------------------------------------------------------
      Case "U"    'UpdatePatient / Changing patient data
      '-------------------------------------------------------------------------------
        Set cTok = New CToken
        With cTok
          'put all data into single token instance
          .TokenID = "U"  'Token identifier


          'Expected data fields / Identification:
          
          '1. Name of patient (old)
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '2. First name of patient (old)
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '3. Date of birth (old)
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '4. External card index number (old)
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          'Expected data fields / Information:
          
          '1. Name of patient (possibly new)
          .LastNameNew = Byte2String(aBytData, lPos, lPos)
          
          '2. First name of patient (possibly new)
          .FirstNameNew = Byte2String(aBytData, lPos, lPos)
          
          '3. Date of birth (possibly new)
          .BirthNew = Byte2String(aBytData, lPos, lPos)
          
          '4. External card index number (possibly new)
          .ExtCardIDNew = Byte2String(aBytData, lPos, lPos)
          
          '5. Sex (possibly new)
          .Sex = Byte2String(aBytData, lPos, lPos)
          
          '6. Permanent dentist (possibly new)
          .PermanentDentistNew = Byte2String(aBytData, lPos, lPos)
          
          '7. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '8. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
  
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With
  
      '-------------------------------------------------------------------------------
      Case "X"    'X-Ray Image / Image order
      '-------------------------------------------------------------------------------
        Set cTok = New CToken
        With cTok
          'put all data into single token instance
          .TokenID = "X"  'Token identifier


          'Expected data fields / Identification:
          
          '1. Order number
          .OrderNo = Byte2String(aBytData, lPos, lPos)
          
          '2. Name of patient
          .LastName = Byte2String(aBytData, lPos, lPos)
          
          '3. First name of patient
          .FirstName = Byte2String(aBytData, lPos, lPos)
          
          '4. Date of birth
          .Birth = Byte2String(aBytData, lPos, lPos)
          
          '5. External card index number
          .ExtCardID = Byte2String(aBytData, lPos, lPos)
          
          'Expected data fields / Information:
          
          '1. Sex
          .Sex = Byte2String(aBytData, lPos, lPos)
          
          '2. Statement on pregnancy
          .pregnant = Byte2String(aBytData, lPos, lPos)
          
          '3. Image type
          .ImageType = Byte2String(aBytData, lPos, lPos)
          
          '4. Image reason/Notes
          .Reason = Byte2String(aBytData, lPos, lPos)
          
          '5. Station name of the calling station
          .CallingStation = Byte2String(aBytData, lPos, lPos)
          
          '6. Date of the order
          .OrderDate = Byte2String(aBytData, lPos, lPos)
          
          '7. Time of the order
          .OrderTime = Byte2String(aBytData, lPos, lPos)
          
          '8. Address of sender
          .AddrOfSender = Byte2String(aBytData, lPos, lPos)
          
          '9. Address of receiver
          .AddrOfReceiver = Byte2String(aBytData, lPos, lPos)
          
          m_ColSlida.Add cTok   '... and add it to our collection now
        End With

      Case Else
         ' ... no other tokens supported
        RaiseEvent InvalidTokenFound
        
    End Select
   
  Loop  'until eof
  
  ReadMailslot = True
  Exit Function
  
ProcErr:
  m_Err = Err       'set error and leave
  Exit Function

End Function

'*****************************************************************************
'
' Function name : Byte2String
'
' Description   : creates VB-string from C-string (0-terminiated).
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : aBytData  = all characters from one line readed
'                 lStart    = where did the last one start
'                 lRetPos   = returns the last position
'
' Returns       : VB-String out of a C-String element
'
' Extern data   : Name                          Access (r/w/&)
'                 m_Convert                     &
'                 m_Err                         w
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Private Function Byte2String(aBytData() As Byte, ByVal lStart As Long, ByRef lRetPos As Long) As String
   Dim ci        As Long     'Counter
   Dim strCon    As String   'Help-String collection the chars
   Dim strAnsi   As String   'ANSI converted String
  
   On Error GoTo ProcErr
  
   ci = lStart
   strCon = ""
  
   Do
      ci = ci + 1
      strCon = strCon & Chr(aBytData(ci))
   Loop While aBytData(ci) <> 0
  
   lRetPos = ci                                  'save pos.
   strCon = m_Convert.StripTerminator(strCon)    'kill the NullChr at the end
   m_Convert.OEM2Ansi strCon, strAnsi            'Convert OEM to Ansi
   Byte2String = RTrim(strAnsi)
   Exit Function

ProcErr:
   m_Err = Err       'set error and exit
   Exit Function

End Function

'*****************************************************************************
'
' Function name : Property Get LastErr
'
' Description   : Retrun last internal error
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : -
'
' Returns       : error code
'
' Extern data   : Name                          Access (r/w/&)
'                 m_Err                         r
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Property Get LastErr() As Long
   LastErr = m_Err
End Property
