VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CStrConv"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'*****************************************************************************
'
' Classname     : CStrConv (Code)
'
' Description   : String Convertions
'                 MS-DOS-(ASCII-)text to Windows text (OEM2Ansi) and reverse
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'
'  2000 Sirona Dental Systems GmbH - All Rights Reserved
'-----------------------------------------------------------------------------
' Status        : Freigegeben
'
' Security      : -
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -      Modification
' 28.01.00 -      Design
'
'*****************************************************************************
Option Explicit

'*****************************************************************************
'Private definitions and constants
'*****************************************************************************

'*****************************************************************************
'Private data types
'*****************************************************************************

'*****************************************************************************
' Global variables and constants
'*****************************************************************************

'---API Declarations BEGIN------------------------------------------------------------
Private Declare Function OemToCharBuff Lib "user32" Alias "OemToCharBuffA" (ByVal lpszSrc As String, ByVal lpszDst As String, ByVal cchDstLength As Long) As Long
Private Declare Function CharToOemBuff Lib "user32" Alias "CharToOemBuffA" (ByVal lpszSrc As String, ByVal lpszDst As String, ByVal cchDstLength As Long) As Long
'---API Declarations END------------------------------------------------------------

'*****************************************************************************
' Implementation
'*****************************************************************************

'*****************************************************************************
'
' Function name : StripTerminator(ByVal strString As String) As String
'
' Description   : Returns a string without any zero terminator.  Typically,
'                 this was a string returned by a Windows API call.
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : ByVal strString As String     / String to remove terminator from
'
' Returns       : The value of the string passed in minus any terminating zero.
'
' Extern data   : -
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Function StripTerminator(ByVal strString As String) As String
    Dim intZeroPos As Integer

    intZeroPos = InStr(strString, Chr$(0))
    
    If intZeroPos > 0 Then
        StripTerminator = Left$(strString, intZeroPos - 1)
    Else
        StripTerminator = strString
    End If

End Function

'*****************************************************************************
'
' Function name : OEM2Ansi
'
' Description   : MS-Dos Text to Windows Text
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : ByVal strIn As String         / Input
'                 ByRef strOut As String        / Returned result
'
' Returns       : -
'
' Extern data   : -
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Sub OEM2Ansi(ByVal strIn As String, ByRef strOut As String)
  strOut = strIn & vbNullChar
  strIn = strIn & vbNullChar
  OemToCharBuff strIn, strOut, Len(strOut)
  strOut = StripTerminator(strOut)  'throw away to 0 at the end
End Sub

'*****************************************************************************
'
' Function name : Ansi2OEM
'
' Description   : Windows Text to MS-DOS-Text
'
' Author        : Klaus-Peter Heinz             Dep.: Fa. MGW
'-----------------------------------------------------------------------------
' Parameter     : ByVal strIn As String         / Input
'                 ByRef strOut As String        / Returned result
'
' Returns       : -
'
' Extern data   : -
'
' Remarks       : -
'-----------------------------------------------------------------------------
' Security      : -
'
' Status        : Design
'-----------------------------------------------------------------------------
' Revision history - Version 1.0
'
' Date     -     Modification
' 28.01.00 -     Design
'
'*****************************************************************************
Public Sub Ansi2OEM(ByVal strIn As String, ByRef strOut As String)
  strOut = strIn & vbNullChar
  strIn = strIn & vbNullChar
  CharToOemBuff strIn, strOut, Len(strOut)
  strOut = StripTerminator(strOut)  'weg mit der 0 am Ende
End Sub
