/******************************************************************************/
/*                                                                            */
/* Headername  : basetypw.h                                                   */
/*                                                                            */
/* Beschreibung: Weitere Basistypen zusaetzlich zu windows.h.                 */
/*                                                                            */
/* Autor       : Juergen Zimmermann            Abt.: Siemens AG UB Med DRe    */
/*                                                                            */
/* Copyright (C) Siemens AG 1991 - 1994 - All Rights Reserved                 */
/* -------------------------------------------------------------------------- */
/* Status      : Freigegeben                                                  */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.0_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 29.07.92 -    Getestet                                                     */
/* 02.03.93 -    Pointertypen mit und ohne const dazu                         */
/* 05.09.94 -    Freigabe                                                     */
/*                                                                            */
/******************************************************************************/
#ifndef _BASETYPW_H
   #define _BASETYPW_H

#ifdef _MAIN_
   #define STORAGE
#else
   #define STORAGE extern
#endif

/******************************************************************************/
/* Datentypen                                                                 */
/******************************************************************************/
#ifndef _UBYTE_
   #define _UBYTE_
   typedef       unsigned     char            UBYTE;
   typedef       unsigned     char      *    PUBYTE;
   typedef       unsigned     char far  *   LPUBYTE;
   typedef const unsigned     char      *   PCUBYTE;
   typedef const unsigned     char far  *  LPCUBYTE;
#endif

#ifndef _SBYTE_
   #define _SBYTE_
   typedef         signed     char            SBYTE;
   typedef         signed     char      *    PSBYTE;
   typedef         signed     char far  *   LPSBYTE;
   typedef const   signed     char      *   PCSBYTE;
   typedef const   signed     char far  *  LPCSBYTE;
#endif

#ifndef _UWORD_
   #define _UWORD_
   typedef       unsigned      int            UWORD;
   typedef       unsigned      int      *    PUWORD;
   typedef       unsigned      int far  *   LPUWORD;
   typedef const unsigned      int      *   PCUWORD;
   typedef const unsigned      int far  *  LPCUWORD;
#endif

#ifndef _SWORD_
   #define _SWORD_
   typedef         signed      int            SWORD;
   typedef         signed      int      *    PSWORD;
   typedef         signed      int far  *   LPSWORD;
   typedef const   signed      int      *   PCSWORD;
   typedef const   signed      int far  *  LPCSWORD;
#endif

#ifndef _UDWORD_
   #define _UDWORD_
   typedef       unsigned long int           UDWORD;
   typedef       unsigned long int      *   PUDWORD;
   typedef       unsigned long int far  *  LPUDWORD;
   typedef const unsigned long int      *  PCUDWORD;
   typedef const unsigned long int far  * LPCUDWORD;
#endif

#ifndef _SDWORD_
   #define _SDWORD_
   typedef         signed long int           SDWORD;
   typedef         signed long int      *   PSDWORD;
   typedef         signed long int far  *  LPSDWORD;
   typedef const   signed long int      *  PCSDWORD;
   typedef const   signed long int far  * LPCSDWORD;
#endif

#ifndef _REAL_
   #define _REAL_
   typedef       float                         REAL;
   typedef       float                  *     PREAL;
   typedef       float             far  *    LPREAL;
   typedef const float                  *    PCREAL;
   typedef const float             far  *   LPCREAL;
#endif

#ifndef _DREAL_
   #define _DREAL_
   typedef       double                       DREAL;
   typedef       double                 *    PDREAL;
   typedef       double            far  *   LPDREAL;
   typedef const double                 *   PCDREAL;
   typedef const double            far  *  LPCDREAL;
#endif

/******************************************************************************/
/* Definitionen, Makros und Konstanten                                        */
/******************************************************************************/
#ifndef _EOS_
   #define _EOS_
   #define EOS 0                           /* Standard C-Stringendemarkierung */
#endif                                                               /* _EOS_ */

#ifdef _MAIN_
           const UBYTE  UBMAX  =          0xFF;
           const SBYTE  SBMAX  =          0x7F;
           const UWORD  UWMAX  =        0xFFFF;
           const SWORD  SWMAX  =        0x7FFF;
           const UDWORD UDWMAX =   0xFFFFFFFFL;
           const SDWORD SDWMAX =   0x7FFFFFFFL;
           const REAL   RMAX   =     3.37E+38F;
           const REAL   RMIN   =     8.43E-37F;
           const DREAL  DRMAX  = 1.797693E+308;
           const DREAL  DRMIN  = 2.225074E-308;
#else
   STORAGE const UBYTE  UBMAX;
   STORAGE const SBYTE  SBMAX;
   STORAGE const UWORD  UWMAX;
   STORAGE const SWORD  SWMAX;
   STORAGE const UDWORD UDWMAX;
   STORAGE const SDWORD SDWMAX;
   STORAGE const REAL   RMAX;
   STORAGE const REAL   RMIN;
   STORAGE const DREAL  DRMAX;
   STORAGE const DREAL  DRMIN;
#endif                                                       /* #ifdef _MAIN_ */

#endif                                                         /* _BASETYPW_H */
