/******************************************************************************/
/*                                                                            */
/* Modulname   : SiUDate.C       *****  *  *   *  ****    ***   *****  *****  */
/*                               *      *  *   *  *   *  *   *    *    *      */
/* Interface   : SiUDate.H       *****  *  *   *  *   *  *****    *    ****   */
/*                                   *  *  *   *  *   *  *   *    *    *      */
/* Resourcen   : SiUtil.RC       *****  *   ***   ****   *   *    *    *****  */
/*                                                                            */
/* Interface   : SiUtil.HRC                                                   */
/*                                                                            */
/* Beschreibung: Die DLL besteht aus einer Reihe global verfuegbarer Utility- */
/*               Funktionen zur Datumsbearbeitung.                            */
/*                                                                            */
/* Autor       : Michael G. Wetzel             Abt.: Siemens AG UB Med DRE    */
/*                                                                            */
/* Copyright (C) Siemens AG 1994 - All Rights Reserved                        */
/* -------------------------------------------------------------------------- */
/* Status      : Freigegeben                                                  */
/*                                                                            */
/* Testoptionen: Siehe siutil.mak                                             */
/*                                                                            */
/* Sicherheit  : -                                                            */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.1_                                         */
/*                                                                            */
/* Datum    - Modifikation                                                    */
/* 06.01.93 - Entwurf                                                         */
/* 08.02.93 - Uhrzeit-Funktionen (MGW)                                        */
/* 02.03.93 - Umstellung auf STRICT und WindowsX.h (MGW)                      */
/* 02.03.93 - Umstellung auf neue Zeigertypen (J. Zimmermann)                 */
/* 27.10.93 - Review zur NULL - Serie     (K. Schrick)                        */
/* 03.12.93 - TimeToStr, DateToStr mit CheckTimeFormat/CheckDateFormat (SK)   */
/* 25.02.94 - SetActDateTime Entwurf (MGW)                                    */
/* 01.06.94 - IsDate() toleriert evtl. Nulldatum (J. Zimmermann)              */
/* 09.06.94 - + CalcDate() (J. Zimmermann)                                    */
/* 07.07.94 - + SetActTimeDate() (M. Reinke), Dialogproz. entfaellt           */
/* 08.07.94 - CalcDate fuer Jahre, Monate und beliebige Tage (MGW)            */
/* 25.08.94 - CalcDate fuer negative Tage, Monate und Jahre (MGW)             */
/* 25.08.94 - IsLeapYear (MGW); Freigabe V.1.0 (Juergen Zimmermann)           */
/* 17.02.95 - + GetDlgItemDateStr() => V.1.1 (Dietmar Keller)                 */
/* 21.02.95 - GetDlgItemDateStr() fuer Nulldatumseingabe erweitert (D.Keller) */
/* 22.02.95 - Review letzte Aenderung (Juergen Zimmermann)                    */
/* 10.03.95 - TimeToStr(), DateToStr(), GetDlgItemDateStr(), StrToDate(),     */
/*            StrToTime() mit Default (Dietmar Keller)                        */
/* 14.03.95 - Review letzte Aenderung (Juergen Zimmermann)                    */
/* 30.06.95 - Review (Dietmar Keller, Fa. Konplan)                            */
/* 03.07.95 - Integrationstest + Freigabe V.1.1 (Juergen Zimmermann)          */
/*                                                                            */
/* -------------------------------------------------------------------------- */
/* Funktion -    Kurzbeschreibung                                             */
/*                                                                            */
/* - public -                                                                 */
/* SetActTimeDate     Setzt das aktuelle  Systemdatum + Systemzeit (Dialog)   */
/* GetActTime         Ermittelt aktuelle  Systemuhrzeit                       */
/* GetActDate         Ermittelt aktuelles Systemdatum                         */
/* GetActTimeDateStr  Ermittelt aktuelles Systemdatum + Systemzeit als String */
/* IsTime             Pruefung auf gueltige Uhrzeit                           */
/* IsDate             Pruefung auf gueltiges Datum                            */
/* CompareTime        Vergleicht Uhrzeit1 und Uhrzeit2                        */
/* CompareDate        Vergleicht Datum1   und Datum2                          */
/* TimeToStr          Wandelt Uhrzeit in String um                            */
/* DateToStr          Wandelt Datum in String um                              */
/* GetDlgItemDateStr  Wandelt Datumeingabe(Editfeld) in formatierten String um*/
/* StrToTime          Wandelt String in Uhrzeit um                            */
/* StrToDate          Wandelt String in Datum um                              */
/* CalcDate           Berechnet ein neues Datum aufgrund von Offsets          */
/*                                                                            */
/* - private -                                                                */
/* CheckDateFormat    Prueft Datumsseparatorzeichen                           */
/* CheckTimeFormat    Prueft Zeitseparatorzeichen                             */
/* IsLeapYear         Prueft auf Schaltjahr                                   */
/*                                                                            */
/* - Callback -                                                               */
/* -                                                                          */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* Standard-Include-Dateien                                                   */
/******************************************************************************/
#define  STRICT
#include <windows.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

/******************************************************************************/
/* Eigene Include-Dateien                                                     */
/******************************************************************************/
#include "basetypw.h"                                     /* Allg. Basistypen */
#include "SiUDate.h"                                      /* Eigene API       */

/******************************************************************************/
/* Private-Definitionen, -Makros und -Konstanten                              */
/******************************************************************************/
#define YEAR_MIN     1880          /* Aeltestes erlaubtes Jahr (Geburtsjahr)  */

#define FORMATSTRLEN   15          /* Maximallaenge Datums-/Zeitformat-String */

/*----------------------------------------------------------------------------*
 * Zeichencharakterisierung Datumsformat                                      *
 *----------------------------------------------------------------------------*/
#define CHARIS_SPACE    0           /* Zeichen = Leertaste                    */
#define CHARIS_DIGIT    1           /* Zeichen = Ziffer                       */
#define CHARIS_FORMT    2           /* Zeichen = Gueltiges Formatzeichen      */
#define CHARIS_WRONG    3           /* Zeichen = Ungueltiges Zeichen          */

/******************************************************************************/
/* Private-Datentypen                                                         */
/******************************************************************************/

/******************************************************************************/
/* Externe Referenzen                                                         */
/******************************************************************************/
STORAGE HINSTANCE  hInstGl;             /* Instance der DLL                   */

/******************************************************************************/
/* Modulglobale Variable                                                      */
/******************************************************************************/
static char szTimeFormatGl[] = ":. ";           /* Formatierungszeichen Zeit  */
static char szDateFormatGl[] = ":./-,";         /* Formatierungszeichen Datum */

/******************************************************************************/
/* Prototypen der private-Funktionen                                          */
/******************************************************************************/
static BOOL CheckTimeFormat( char  chTimeFormat );
static BOOL CheckDateFormat( char  chDateFormat );
static BOOL IsLeapYear     ( UWORD uwYear       );

/******************************************************************************/
/* Implementierung                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* Funktionsname: GetActTime                                                  */
/*                                                                            */
/* Beschreibung : Gibt die aktuelle Zeit zurueck                              */
/*                                                                            */
/* Autor        : Michael G. Wetzel            Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : lpubHour   = Puffer fuer die Stunde                         */
/*                lpubMinute = Puffer fuer die Minute                         */
/*                lpubSecond = Puffer fuer die Sekunde                        */
/*                                                                            */
/* Rueckgabewert: -                                                           */
/*                                                                            */
/* Externdaten  : -                                                           */
/*                                                                            */
/* Bemerkungen  : -                                                           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version 01.00                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 08.02.93 -    Version 1.00                                                 */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/*                                                                            */
/******************************************************************************/
void WINAPI GetActTime(LPUBYTE lpubHour, LPUBYTE lpubMinute, LPUBYTE lpubSecond)
{
   static struct tm *ptm;      /* Hilfsvariable (static wg. DOS-Funkt. & DLL) */
   static time_t     timer;
    
   (void) time( &timer );
   ptm = localtime( &timer );

   *lpubHour   = (UBYTE) (ptm->tm_hour);
   *lpubMinute = (UBYTE) (ptm->tm_min );
   *lpubSecond = (UBYTE) (ptm->tm_sec );
}                                      

/******************************************************************************/
/*                                                                            */
/* Funktionsname: GetActDate                                                  */
/*                                                                            */
/* Beschreibung : Gibt das aktuelles Datum zurueck                            */
/*                                                                            */
/* Autor        : J. Zimmermann                Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : lpuwYear  = Puffer fuer das Jahr                            */
/*                lpubMonth = Puffer fuer den Monat                           */
/*                lpubDay   = Puffer fuer den Tag                             */
/*                                                                            */
/* Rueckgabewert: -                                                           */
/*                                                                            */
/* Externdaten  : -                                                           */
/*                                                                            */
/* Bemerkungen  : -                                                           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.0_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 05.01.93 -    Entwurf                                                      */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/*                                                                            */
/******************************************************************************/
void WINAPI GetActDate( LPUWORD lpuwYear, LPUBYTE lpubMonth, LPUBYTE lpubDay )
{
   static struct tm *ptm;      /* Hilfsvariable (static wg. DOS-Funkt. & DLL) */
   static time_t     timer;

   (void) time( &timer );
   ptm = localtime( &timer );

   *lpuwYear  = (UWORD) (ptm->tm_year + 1900 );
   *lpubMonth = (UBYTE) (ptm->tm_mon  + 1    );
   *lpubDay   = (UBYTE) (ptm->tm_mday        );    
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: GetActDateTimeStr                                           */
/*                                                                            */
/* Beschreibung : Gibt aktuelles Datum und Uhrzeit als String zurueck         */
/*                                                                            */
/* Autor        : MGW                          Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : lpszDateTime   = Puffer fuer den Rueckgabestring            */
/*                lpszFormatDate = Format-String (Y,M,D als Bezeichner),      */ 
/*                                 z.B. "YYYY.MM.DD"                          */
/*                               = NULL => Internes landesspezifisches Format */
/*                lpszFormatTime = Format-String (H,M,S als Bezeichner),      */ 
/*                                 z.B. "HH:MM:SS"                            */
/*                               = NULL => Internes landesspezifisches Format */
/*                                                                            */
/* Rueckgabewert: FALSE im Fehlerfall, sonst TRUE.                            */
/*                                                                            */
/* Externdaten  : -                                                           */
/*                                                                            */
/* Bemerkungen  : Im Fehlerfall wird lpszDateTime mit EOS initialisiert.      */
/*                Zuerst wird das Datum, dann die Uhrzeit angegeben.          */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.0_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 10.08.93 -    Entwurf (MGW)                                                */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI GetActTimeDateStr( LPSTR  lpszDateTime, LPCSTR lpszDateFormat,
                               LPCSTR lpszTimeFormat )
{ 
    UWORD uwYear   = 0;
    UBYTE ubMonth  = 0;
    UBYTE ubDay    = 0;
    UBYTE ubHour   = 0;
    UBYTE ubMinute = 0;
    UBYTE ubSecond = 0;
    
    GetActDate( &uwYear, &ubMonth , &ubDay    );
    GetActTime( &ubHour, &ubMinute, &ubSecond ); 
    
    if (!DateToStr( uwYear, ubMonth , ubDay, lpszDateTime, lpszDateFormat ) )
       return( FALSE );
              
    (void) lstrcat( lpszDateTime, " " );
    
    return( TimeToStr( ubHour, ubMinute, ubSecond, 
                     &lpszDateTime[lstrlen( lpszDateTime )], lpszTimeFormat ) );
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: IsTime                                                      */
/*                                                                            */
/* Beschreibung : Prueft ob die angegebene Kombination aus Stunde, Minute und */
/*                Sekunde eine gueltige Uhrzeit darstellt.                    */
/*                                                                            */
/* Autor        : Michael G. Wetzel            Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : ubHour   = Stunde                                           */
/*                ubMinute = Minute                                           */
/*                ubSecond = Sekunde                                          */
/*                                                                            */
/* Rueckgabewert: TRUE wenn gueltige Zeit, sonst FALSE                        */
/*                                                                            */
/* Bemerkungen  : -                                                           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version 01.00                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 08.02.93      Entwurf                                                      */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI IsTime( UBYTE ubHour, UBYTE ubMinute, UBYTE ubSecond )
{
   if (ubHour   > 23) return( FALSE );
   if (ubMinute > 59) return( FALSE );
   if (ubSecond > 59) return( FALSE );
   return( TRUE );
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: IsDate                                                      */
/*                                                                            */
/* Beschreibung : Prueft ob die angegebene Kombination aus Jahr, Monat und Tag*/
/*                ein gueltiges Datum darstallt.                              */
/*                                                                            */
/* Autor        : Michael G. Wetzel            Abt.: Siemens AG UB Med DRE    */
/* -------------------------------------------------------------------------- */
/* Parameter    : uwYear      = Jahr                                          */
/*                ubMonth     = Monat                                         */
/*                ubDay       = Tag                                           */
/*                fIgnoreZero = TRUE  => Nulldatum (0.0.00) ist erlaubt       */
/*                            = FALSE => Nulldatum ist ungueltig              */
/*                                                                            */
/* Rueckgabewert: TRUE wenn gueltiges Datum, sonst FALSE                      */
/*                                                                            */
/* Bemerkungen  : Schaltjahrberechnung gilt vermutlich nur bis 1.1.2050 (wird */
/*                nicht geprueft).                                            */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.0_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 19.10.92 -    Entwurf                                                      */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/* 08.11.93 -    Jahr groesser als Minimum, kleiner als Aktuelles             */
/* 01.06.94 -    Nulldatum erlaubt                                            */
/* 22.02.95 -    Beschreibungskorrektur (Juergen Zimmermann)                  */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI IsDate( UWORD uwYear, UBYTE ubMonth, UBYTE ubDay, BOOL fIgnoreZero )
{
   UWORD uwActYear;                                        /* Aktuelles Jahr  */
   UBYTE ubActMonth;                                       /* Aktueller Monat */
   UBYTE ubActDay;                                         /* Aktueller Tag   */
   
   /*-------------------------------------------------------------------------*
    * Auf Nulldatum pruefen und evtl. tolerieren                              *
    *-------------------------------------------------------------------------*/
   if (!uwYear && !ubMonth && !ubDay && fIgnoreZero) return( TRUE );
   
   /*-------------------------------------------------------------------------*
    * Datum darf nicht zu alt und nicht zu neu (> aktuelles) sein             *
    *-------------------------------------------------------------------------*/
   GetActDate( &uwActYear, &ubActMonth , &ubActDay );
   if (   uwYear < YEAR_MIN 
       || CompareDate( uwYear, ubMonth, ubDay, uwActYear, ubActMonth, ubActDay ) 
          == 1) return( FALSE );

   /*-------------------------------------------------------------------------*
    * Tagesanzahl abpruefen                                                   *
    *-------------------------------------------------------------------------*/
   switch (ubMonth)
   {
      case  1:                                           /* Januar            */
      case  3:                                           /* Maerz             */
      case  5:                                           /* Mai               */
      case  7:                                           /* Juli              */
      case  8:                                           /* August            */
      case 10:                                           /* Oktober           */
      case 12:                                           /* Dezember          */
         if (ubDay < 1 || ubDay > 31) return( FALSE );
         break;

      case  4:                                           /* April             */
      case  6:                                           /* Juni              */
      case  9:                                           /* September         */
      case 11:                                           /* November          */
         if (ubDay < 1 || ubDay > 30) return( FALSE );
         break;

      case  2:                                           /* Februar           */
         if ( IsLeapYear(uwYear) )                       /* Schaltjahr        */
         {
            if (ubDay < 1 || ubDay > 29) return( FALSE );
         }
         else                                            /* Kein Schaltjahr   */
         {      
            if (ubDay < 1 || ubDay > 28) return( FALSE );
         }
         break;

      default:                                           /* Ungueltiger Monat */
         return( FALSE );
   }
   return( TRUE );
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: CompareTime                                                 */
/*                                                                            */
/* Beschreibung : Vergleicht 2 Uhrzeiteintraege miteinander.                  */
/*                                                                            */
/* Autor        : Michael G. Wetzel            Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : ubHour1   = Stunde  von Uhrzeit 1                           */
/*                ubMinute1 = Minute  von Uhrzeit 1                           */
/*                ubSecond1 = Sekunde von Uhrzeit 1                           */
/*                ubHour2   = Stunde  von Uhrzeit 2                           */
/*                ubMinute2 = Minute  von Uhrzeit 2                           */
/*                ubSecond2 = Sekunde von Uhrzeit 2                           */
/*                                                                            */
/* Rueckgabewert: -1 <=> Uhrzeit 1 < Uhrzeit 2                                */
/*                 0 <=> Uhrzeit 1 = Uhrzeit 2                                */
/*                +1 <=> Uhrzeit 1 > Uhrzeit 2                                */
/*                                                                            */
/* Externdaten  : -                                                           */
/*                                                                            */
/* Bemerkungen  : -                                                           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.0_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 08.02.93 -    Version 1.00                                                 */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/*                                                                            */
/******************************************************************************/
SWORD WINAPI CompareTime( UBYTE ubHour1, UBYTE ubMinute1, UBYTE ubSecond1,
                          UBYTE ubHour2, UBYTE ubMinute2, UBYTE ubSecond2 )
{
   /*-------------------------------------------------------------------------*/
   /* Stunden vergleichen                                                     */
   /*-------------------------------------------------------------------------*/
   if (ubHour1   < ubHour2  ) return( -1 );               /* Stunde kleiner   */
   
   if (ubHour1   > ubHour2  ) return( +1 );               /* Stunde groesser  */

   /*-------------------------------------------------------------------------*/
   /* Stunde gleich => Minuten vergleichen                                    */
   /*-------------------------------------------------------------------------*/
   if (ubMinute1 < ubMinute2) return( -1 );               /* Minute kleiner   */
   
   if (ubMinute1 > ubMinute2) return( +1 );               /* Minute groesser  */

   /*-------------------------------------------------------------------------*/
   /* Stunde und Minute gleich => Sekunde vergleichen                         */
   /*-------------------------------------------------------------------------*/
   if (ubSecond1 < ubSecond2) return( -1 );               /* Sekunde kleiner  */
   
   if (ubSecond1 > ubSecond2) return( +1 );               /* Sekunde groesser */

   /*-------------------------------------------------------------------------*/
   /* Alles gleich                                                            */
   /*-------------------------------------------------------------------------*/
   return( 0 );
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: CompareDate                                                 */
/*                                                                            */
/* Beschreibung : Vergleicht 2 Datumeintraege miteinander.                    */
/*                                                                            */
/* Autor        : J. Zimmermann                Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : uwYear1  = Jahr  von Datum 1                                */
/*                ubMonth1 = Monat von Datum 1                                */
/*                ubDay1   = Tag   von Datum 1                                */
/*                uwYear2  = Jahr  von Datum 2                                */
/*                ubMonth2 = Monat von Datum 2                                */
/*                ubDay2   = Tag   von Datum 2                                */
/*                                                                            */
/* Rueckgabewert: -1 <=> Datum 1 < Datum 2                                    */
/*                 0 <=> Datum 1 = Datum 2                                    */
/*                +1 <=> Datum 1 > Datum 2                                    */
/*                                                                            */
/* Externdaten  : -                                                           */
/*                                                                            */
/* Bemerkungen  : -                                                           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.0_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 06.01.93 -    Entwurf                                                      */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/*                                                                            */
/******************************************************************************/
SWORD WINAPI CompareDate( UWORD uwYear1, UBYTE ubMonth1, UBYTE ubDay1,
                          UWORD uwYear2, UBYTE ubMonth2, UBYTE ubDay2 )
{
   /*-------------------------------------------------------------------------*/
   /* Jahre vergleichen                                                       */
   /*-------------------------------------------------------------------------*/
   if (uwYear1  < uwYear2 ) return( -1 );                   /* Jahr kleiner   */
   
   if (uwYear1  > uwYear2 ) return( +1 );                   /* Jahr groesser  */

   /*-------------------------------------------------------------------------*/
   /* Jahr gleich => Monate vergleichen                                       */
   /*-------------------------------------------------------------------------*/
   if (ubMonth1 < ubMonth2) return( -1 );                   /* Monat kleiner  */
   
   if (ubMonth1 > ubMonth2) return( +1 );                   /* Monat groesser */

   /*-------------------------------------------------------------------------*/
   /* Jahr und Monat gleich => Tag vergleichen                                */
   /*-------------------------------------------------------------------------*/
   if (ubDay1   < ubDay2  ) return( -1 );                   /* Tag kleiner    */
   
   if (ubDay1   > ubDay2  ) return( +1 );                   /* Tag groesser   */

   /*-------------------------------------------------------------------------*/
   /* Alles gleich                                                            */
   /*-------------------------------------------------------------------------*/
   return( 0 );
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: TimeToStr                                                   */
/*                                                                            */
/* Beschreibung : Uhrzeit in String konvertieren                              */
/*                                                                            */
/* Autor        : Michael G. Wetzel            Abt.: Siemens AG UB Med DRE    */
/* -------------------------------------------------------------------------- */
/* Parameter    : ubHour     = Stunde                                         */
/*                ubMinute   = Minute                                         */
/*                ubSecond   = Sekunde                                        */
/*                lpszTime   = Buffer fuer generierten Uhrzeit-String         */
/*                lpszFormat = Format-String (H,M,S als Bezeichner),          */ 
/*                             z.B. "HH:MM:SS"                                */
/*                           = NULL => Internes landesspezifisches Format     */
/*                                                                            */
/* Rueckgabewert: TRUE wenn OK sonst FALSE                                    */
/*                Der Zeit-String wird im Fehlerfall mit EOS initialisiert    */
/*                                                                            */
/* Externdaten  : Name                         Zugriff (r/w/&)                */
/*                hDLLInstGl                   r                              */
/*                                                                            */
/* Bemerkungen  : -                                                           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.1_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 18.02.93 -    Entwurf                                                      */ 
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/* 03.12.93 -    Funktion robuster durch Initialisierung von lpszTime,        */
/*               Ueberpruefung von Formatstringpos. cb und CheckTimeFormat, SK*/
/* 10.03.95 -    + IDS_TIMEFORMAT (Dietmar Keller) => V.1.1                   */
/* 14.03.95 -    Review letzte Aenderung (Juergen Zimmermann)                 */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI TimeToStr( UBYTE ubHour, UBYTE ubMinute, UBYTE ubSecond,
                       LPSTR lpszTime, LPCSTR lpszFormat )
{
   UBYTE cb;                       /* Aktuelle Position in Formatstring       */
   UBYTE ubPos;                    /* Position letztes Zeichen eines Formates */
   UBYTE ubParamCount;             /* Zaehler fuer Format-Parameter           */
   int   iPAR[3];                  /* Uebergabeparameter bei wsprintf         */
   char  szPFORMAT[FORMATSTRLEN];  /* Formatstring fuer wsprintf              */
   char  szFormat [FORMATSTRLEN];  /* Zeit-Formatstring                       */

   cb           = 0;
   ubParamCount = 0;
   lpszTime [0] = EOS;                    /* Zielstring init. fuer Fehlerfall */
   szPFORMAT[0] = EOS;


   /*-------------------------------------------------------------------------*/
   /* Evtl. Default benutzen                                                  */
   /*-------------------------------------------------------------------------*/
   if (!lpszFormat)
   {
      return( FALSE );
   }
   else 
   {
      (void) lstrcpy( szFormat, lpszFormat );
   }
     
   if (lstrlen( szFormat ) > 8)         /* Format eg. "HH:MM:SS" ueberpruefen */
   {
     return ( FALSE );                  /* kein korrekter Formatstring        */
   }
   
   while ( (szFormat[cb] != EOS) &&     /* Formatvorlage nicht Ende           */
           (ubParamCount < 3)      &&   /* Anzahl Formatparameter OK          */
           (cb <= FORMATSTRLEN - 5) )   /* aktuelle Formatpos. nicht Ende     */
                                        /* -5 wg. lstrcat ( "%02d" )          */
   {
      switch (szFormat[cb])
      {
         /*-------------------------------------------------------------------*/
         /* Stunde bearbeiten                                                 */
         /*-------------------------------------------------------------------*/
         case 'H':
         case 'h':

            for (ubPos = 0; toupper( szFormat[cb] ) == 'H'; ubPos++, cb++);

            if (ubPos == 2)                   /* 2-stellige Angabe erwuenscht */
            {
               iPAR[ubParamCount] = (ubHour % 24);
               (void) lstrcat( szPFORMAT, "%02d" );
            }
            else
               return( FALSE );               /* Nicht 2-stellig              */
            
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Minute bearbeiten                                                 */
         /*-------------------------------------------------------------------*/
         case 'M':
         case 'm':

            for (ubPos = 0; toupper( szFormat[cb] ) == 'M'; ubPos++, cb++);

            if (ubPos == 2)                   /* 2-stellige Angabe erwuenscht */
            {
               iPAR[ubParamCount] = (ubMinute % 60);
               (void) lstrcat( szPFORMAT, "%02d" );
            }
            else
               return( FALSE );               /* Nicht 2-stellig              */
            
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Sekunde bearbeiten                                                */
         /*-------------------------------------------------------------------*/
         case 'S':
         case 's':

            for (ubPos = 0; toupper( szFormat[cb] ) == 'S'; ubPos++, cb++);

            if (ubPos == 2)                   /* 2-stellige Angabe erwuenscht */
            {
               iPAR[ubParamCount] = (ubSecond % 60);
               (void) lstrcat( szPFORMAT, "%02d" );
            }
            else
               return( FALSE );               /* Nicht 2-stellig              */
            
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Weder H, M, S gefunden => Format-Zeichen in Ausgabe kopieren      */
         /*-------------------------------------------------------------------*/
         default:
            
            if (CheckTimeFormat( szFormat[cb] ) )
            {
              szPFORMAT[lstrlen( szPFORMAT ) + 1] = EOS;
              szPFORMAT[lstrlen( szPFORMAT )    ] = szFormat[cb];
              cb++;                           /* Formatindex erhoehen         */
            }
            else
               return( FALSE );               /* kein gueltiges Formatzeichen */
            
            break;
      }
   }                                          
   
   (void) wsprintf( lpszTime, szPFORMAT, iPAR[0], iPAR[1], iPAR[2] );
   return( TRUE );
}                                                         /* Ende TimeToStr() */

/******************************************************************************/
/*                                                                            */
/* Funktionsname: DateToStr                                                   */
/*                                                                            */
/* Beschreibung : Datum in String konvertieren                                */
/*                                                                            */
/* Autor        : Hanmen /MGW                  Abt.: Siemens AG UB Med DRE    */
/* -------------------------------------------------------------------------- */
/* Parameter    : uwYear     = Jahr                                           */
/*                ubMonth    = Monat                                          */
/*                ubDay      = Tag                                            */
/*                lpszDate   = Buffer fuer generierten Datums-String          */
/*                lpszFormat = Format-String (Y,M,D als Bezeichner),          */ 
/*                             z.B. "YYYY.MM.DD"                              */
/*                            = NULL => Internes landesspezifisches Format    */
/*                                                                            */
/* Rueckgabewert: TRUE wenn OK sonst FALSE                                    */
/*                                                                            */
/* Externdaten  : Name                         Zugriff (r/w/&)                */
/*                hDLLInstGl                   r                              */
/*                                                                            */
/* Bemerkungen  : Der Datums-String wird im Fehlerfall mit EOS initialisiert. */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.1_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 19.10.92 -    Entwurf                                                      */
/* 19.10.92 -    Funktion fuer Klein- und Grossbuchstaben im Format (MGW)     */
/* 19.10.92 -    Funktion fuer mehrere Trennzeichen (MGW)                     */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/* 03.12.93 -    Funktion robuster durch Initialisierung von lpstDate         */
/*               Ueberpruefung von Formatstringpos. cb und CheckDateFormat, SK*/
/* 10.03.95 -    + IDS_DATEFORMAT (Dietmar Keller) => V.1.1                   */
/* 14.03.95 -    Review letzte Aenderung (Juergen Zimmermann)                 */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI DateToStr( UWORD uwYear, UBYTE ubMonth, UBYTE ubDay,
                       LPSTR lpszDate, LPCSTR lpszFormat )
{
   UBYTE cb;                       /* aktuelle Position in Formatstring       */
   UBYTE ubPos;                    /* Position letztes Zeichen eines Formates */
   UBYTE ubParamCount;             /* Zaehler fuer Format-Parameter           */
   int   iPAR[3];                  /* Uebergabeparameter bei wsprintf         */
   char  szPFORMAT[FORMATSTRLEN];  /* Formatstring fuer wsprintf              */
   char  szFormat[FORMATSTRLEN];   /* Datums-Formatstring                     */

   cb           = 0;
   ubParamCount = 0;
   lpszDate[0]  = EOS;
   szPFORMAT[0] = EOS;

   /*-------------------------------------------------------------------------*/
   /* Evtl. Default benutzen                                                  */
   /*-------------------------------------------------------------------------*/
   if (!lpszFormat)
   {
       return( FALSE );
   }
   else
      (void) lstrcpy( szFormat, lpszFormat );
      
   if (lstrlen( szFormat ) > 10)      /* Format eg. "DD.MM.YYYY" ueberpruefen */
     return ( FALSE );                         /* kein korrekter Formatstring */

   while ( (szFormat[cb] != EOS) &&               /* Formatvorlage nicht Ende */
           (ubParamCount < 3)      &&            /* Anzahl Formatparameter OK */
           (cb <= FORMATSTRLEN - 5) )       /* aktuelle Formatpos. nicht Ende */
                                            /* -5 wg. lstrcat ( "%02d" )      */
   {
      switch (szFormat[cb])
      {
         /*-------------------------------------------------------------------*/
         /* Jahr bearbeiten                                                   */
         /*-------------------------------------------------------------------*/
         case 'Y':
         case 'y':
            
            for (ubPos = 0; toupper( szFormat[cb] ) == 'Y'; ubPos++, cb++)
               ;

            if (ubPos == 4)                   /* 4-stellige Angabe erwuenscht */
            {
               iPAR[ubParamCount] = uwYear % 10000;
               (void) lstrcat( szPFORMAT, "%04d" );
            }
            else
            {
               if (ubPos == 2)                /* 2-stellige Angabe erwuenscht */
               {
                  iPAR[ubParamCount] = (uwYear % 100);
                  (void) lstrcat( szPFORMAT, "%02d" );
               }
               else
                  return( FALSE );            /* Nicht 4 oder 2-stellig       */
            }
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Monat bearbeiten                                                  */
         /*-------------------------------------------------------------------*/
         case 'M':
         case 'm':

            for (ubPos = 0; toupper( szFormat[cb] ) == 'M'; ubPos++, cb++)
               ;

            if (ubPos == 2)                   /* 2-stellige Angabe erwuenscht */
            {
               iPAR[ubParamCount] = (ubMonth % 13);
               (void) lstrcat( szPFORMAT, "%02d" );
            }
            else
               return( FALSE );                /* Nicht 2-stellig             */

            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Tag bearbeiten                                                    */
         /*-------------------------------------------------------------------*/
         case 'D':
         case 'd':
            
            for (ubPos = 0; toupper( szFormat[cb] ) == 'D'; ubPos++, cb++)
               ;

            if (ubPos == 2)                   /* 2-stellige Angabe erwuenscht */
            {
               iPAR[ubParamCount] = (ubDay % 32);
               (void) lstrcat( szPFORMAT, "%02d" );
            }
            else
               return( FALSE );               /* Nicht 2-stellig              */
            
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Weder Y, M, D gefunden => Format-Zeichen in Ausgabe kopieren      */
         /*-------------------------------------------------------------------*/
         default:

            if (CheckDateFormat( szFormat[cb] ) )
            {
               szPFORMAT[lstrlen( szPFORMAT ) + 1] = EOS;
               szPFORMAT[lstrlen( szPFORMAT )    ] = szFormat[cb];
               cb++;                                  /* Formatindex erhoehen */
            }
            else
               return( FALSE );               /* kein gueltiges Formatzeichen */

            break;
      }
   }
   (void) wsprintf( lpszDate, szPFORMAT, iPAR[0], iPAR[1], iPAR[2] );
   return( TRUE );
}                                                         /* Ende DateToStr() */

/******************************************************************************/
/*                                                                            */
/* Funktionsname: GetDlgItemDateStr                                           */
/*                                                                            */
/* Beschreibung : Gibt die Datumseingabe (Editfeld) als formatierten String   */
/*                zurueck.                                                    */
/*                                                                            */
/* Autor        : D.Keller, Fa. Konplan        Abt.: Siemens AG UB Med DRE    */
/* -------------------------------------------------------------------------- */
/* Parameter    : hDlg           = Handle der Dialogbox                       */
/*                uwIdCtrl       = Control-ID des Text-/Editfeldes            */
/*                lpszDate       = Puffer fuer den Rueckgabestring            */
/*                lpszFormat     = Format-String (Y,M,D als Bezeichner),      */ 
/*                                 z.B. "YYYY.MM.DD"                          */
/*                               = NULL => Internes landesspezifisches Format */
/*                fIgnoreZero    = TRUE  => Nulldatum (0.0.00) ist erlaubt    */
/*                               = FALSE => Nulldatum ist ungueltig           */
/*                                                                            */
/* Rueckgabewert: FALSE : keine Aenderung --> Text im Editfeld == lpszDate    */
/*                        oder Fehler im Format-String.                       */
/*                TRUE  : Der Text im Editfeld wird frisch formatiert ueber   */
/*                        den Parameter lpszDate zurueckgegeben!              */
/*                                                                            */
/* Externdaten  : Name                         Zugriff (r/w/&)                */
/*                hDLLInstGl                   r                              */
/*                                                                            */
/* Bemerkungen  : Im Fehlerfall wird lpszDate mit EOS initialisiert.          */
/*                ' ' gilt je nach Position als 0 oder als Formatzeichen.     */
/*                                                                            */
/*                Aufrufbsp. innerhalb der Dialogprozedur, case WM_COMMAND:   */
/*                                                                            */
/*                if (wParam == uwIdCtrl)                                     */
/*                {                                                           */
/*                   if (HIWORD( lParam ) == EN_UPDATE)                       */
/*                   {                                                        */
/*                      szdText[11];                                          */
/*                                                                            */
/*                      if (GetDlgItemDateStr( hDlg, uwIdCtrl, szdText,       */
/*                                             "DD.MM.YYYY", TRUE ) )         */
/*                      {                                                     */
/*                         SetWindowText( hWndCtrl, szdText );                */
/*                         SendDlgItemMessage( hDlg, uwIdCtrl, WM_KEYDOWN,    */
/*                                                             VK_END, 0 );   */
/*                      }                                                     */
/*                   }                                                        */
/*                   return( FALSE );                                         */
/*                }                                                           */
/*                Hinweis: SetWindowText() nur bei Aenderung - sonst Stapel-  */
/*                         uberlauf! Danach Cursor an Textende setzen.        */
/*                Die String-Puffer muessen mindestens 11 Zeichen lang sein!  */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.0_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 16.02.95 -    Entwurf                                                      */
/* 22.02.95 -    Review (Juergen Zimmermann)                                  */
/* 13.03.95 -    + IDS_DATEFORMAT (Dietmar Keller)                            */
/* 14.03.95 -    Review letzte Aenderung (Juergen Zimmermann)                 */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI GetDlgItemDateStr( HWND hDlg, UINT uwIdCtrl, LPSTR lpszDate, 
                               LPCSTR lpszFormat, BOOL fIgnoreZero )
{
   UBYTE cubParams    = 0;         /* Zaehler fuer Format-Parameter           */
   UBYTE ubDatePos;                /* Position des aktuellen Datums-Zeichens  */
   UBYTE ubFormatPos  = 0;         // Position des aktuellen Format-Zeichens
   UBYTE cubDateLen;               /* Aktuelle Laenge des Datumstrings        */
   UBYTE cubFormatLen;             /* Aktuelle Laenge des Formatstrings       */
   UBYTE ubCharIs;                 /* Ergebnis Zeichentest (CHARIS_...)       */
   UBYTE cub          = 0;         /* Zeichen-Zaehler (Laufvariable)          */
   char  chLastBez    = ' ';       /* Letztes bearbeitetes Bezeichner-Zeichen */
   UWORD uwYear       = 0;         /* Aktuelles Jahr                          */
   UBYTE ubDummy1     = 0;         /* Hilfsvariable fuer GetActDate() u. Check*/
   UBYTE ubDummy2     = 0;         /* Hilfsvariable fuer GetActDate() u. Check*/
   UBYTE ubYear[4];                /* Ziffern der aktuellen Jahreszahl        */
   UBYTE ubPreY[2];                /* Ziffern des vorigen Jahrhunderts        */
   char  szd[11];                  /* Datumspuffer                            */
   char  szFormat[FORMATSTRLEN];   /* Datums-Formatstring                     */
    
   /*-------------------------------------------------------------------------*
    * Evtl. Default benutzen                                                  *
    *-------------------------------------------------------------------------*/
   if (!lpszFormat)
   {
       return( FALSE );
   }
   else
      (void) lstrcpy( szFormat, lpszFormat );
      
   /*-------------------------------------------------------------------------*
    * Parameter-Check, Datumseingabe holen                                    *
    *-------------------------------------------------------------------------*/
   if (!lpszDate || !IsWindow( hDlg ) ) return( FALSE );

   lpszDate[0] = EOS;
   cubFormatLen = (UBYTE) lstrlen( szFormat );

   if (cubFormatLen > 10) return( FALSE );    /* Kein korrekter Format-String */

   cubDateLen = (UBYTE) GetDlgItemText( hDlg, uwIdCtrl, lpszDate,
                                        cubFormatLen + 1 );
   if (!cubDateLen) return( FALSE );

   /*-------------------------------------------------------------------------*
    * Ziffern der aktuellen Jahreszahl ermitteln                              *
    *-------------------------------------------------------------------------*/
   GetActDate( &uwYear, &ubDummy1 , &ubDummy2 );
   ubYear[3] = (UBYTE) (  uwYear %    10        );
   ubYear[2] = (UBYTE) ( (uwYear %   100) /   10);
   ubYear[1] = (UBYTE) ( (uwYear %  1000) /  100);
   ubYear[0] = (UBYTE) ( (uwYear % 10000) / 1000); 

   /*-------------------------------------------------------------------------*
    * Voriges Jahr ermitteln                                                  *
    *-------------------------------------------------------------------------*/
   if (!ubYear[1])
   {
      ubPreY[0] = (UBYTE) (ubYear[0] - 1);
      ubPreY[1] = 9;
   }
   else
   {
      ubPreY[0] = ubYear[0];
      ubPreY[1] = (UBYTE) (ubYear[1] - 1);
   } 
   /*-------------------------------------------------------------------------*
    * Auswerteschleife ueber Format- und Eingabe-String                       *
    *-------------------------------------------------------------------------*/
   while (   szFormat[cub] != EOS                 /* Formatvorlage nicht Ende */
          && cubParams       <= 3                /* Anzahl Formatparameter ok */
          && lpszDate[cub]   != EOS)   /* Datumstringende noch nicht erreicht */
   {   
      /*----------------------------------------------------------------------*
       * Zeichenart ermitteln                                                 *
       *----------------------------------------------------------------------*/
      ubCharIs = CHARIS_WRONG;
      if (lpszDate[cub] == ' '             ) ubCharIs = CHARIS_SPACE;
      if (isdigit( lpszDate[cub] )         ) ubCharIs = CHARIS_DIGIT;
      if (CheckDateFormat( lpszDate[cub] ) ) ubCharIs = CHARIS_FORMT;

      if (ubCharIs == CHARIS_WRONG)       /* Ungueltiges Zeichen) => Loeschen */
      {  
         for (ubDatePos = cub; ubDatePos < cubDateLen; ubDatePos++)
            lpszDate[ubDatePos] = lpszDate[ubDatePos + 1];

         cubDateLen--;       
      }
      else                                               /* Gueltiges Zeichen */
      {
         switch (szFormat[cub])                    /* Datum ins Format setzen */
         {
            /*----------------------------------------------------------------*
             * Jahr bearbeiten                                                *
             *----------------------------------------------------------------*/
            case 'Y':
            case 'y':
               /*-------------------------------------------------------------*
                * Merker ueber die Format-String-Analyse aktualisieren        *
                * (Wievielte Stelle innerhalb des Unterformats?)              *
                *-------------------------------------------------------------*/
               ubFormatPos++;
               
               if (chLastBez != 'Y')    /* Kommt Jahresformat zum 1. Mal vor? */
               {
                  chLastBez = 'Y';
                  cubParams++;
                  ubFormatPos = 1;
               }
               switch (ubFormatPos)     /* Jahresziffer positionsabh. pruefen */
               {    
                  /*----------------------------------------------------------*
                   * 1. Jahresziffer pruefen                                  *
                   *----------------------------------------------------------*/
                  case 1:

                     if (ubCharIs == CHARIS_FORMT) /* Format an falscher Pos.?*/
                     {
                        lpszDate[cub] = 'A';/*..damit naechstes Mal loeschbar */
                        ubFormatPos--;
                        cub--;
                        break;           /* im naechsten Durchgang raus damit */
                     }
                     if (ubCharIs == CHARIS_SPACE)/* ' ' als 0 interpretieren */
                        lpszDate[cub] = '0';
                     break;                       /* Von case 1. Jahresziffer */
                     
                  /*----------------------------------------------------------*
                   * 2. Jahresziffer pruefen                                  *
                   *----------------------------------------------------------*/
                  case 2:

                     if (ubCharIs != CHARIS_DIGIT)
                     {       /* ...dann repraesentiert die 1. Ziffer das Jahr */
                        lpszDate[cub] = lpszDate[cub - 1];
                        lpszDate[cub - 1] = '0';          /* Z.B. "1" => "01" */
                     }
                     /*-------------------------------------------------------*
                      * Soll die Jahreszahl auf 4 Ziffern expandiert werden?  *
                      *-------------------------------------------------------*/
                     if (    (   lpszDate[cub - 1] != '0' + ubYear[0]
                                  /* Erste 2 Ziffern nicht dieses Jahrhundert */
                              || lpszDate[cub    ] != '0' + ubYear[1])
                         &&  (   lpszDate[cub - 1] != '0' + ubPreY[0]
                                 /* Erste 2 Ziffern nicht voriges Jahrhundert */
                              || lpszDate[cub    ] != '0' + ubPreY[1])
                         && !(   fIgnoreZero && lpszDate[cub] == '0'
                             /* Erste 2 Ziffern != 0, falls Nulldatum erlaubt */
                              && lpszDate[cub - 1] == '0'           )
                                 /* Anzahl Jahresziffern im Format-String > 2 */
                         && toupper( szFormat[cub + 1] ) == 'Y'     )
                     {
                        cubDateLen += 2;
                        if (cubDateLen > cubFormatLen)
                           cubDateLen = cubFormatLen;      /* Vorsichtshalber */

                        /* Ab Jahresziffern alles um 2 Stellen nach rechts    */                
                        for(ubDatePos = cubDateLen; ubDatePos > cub;ubDatePos--)
                           lpszDate[ubDatePos] = lpszDate[ubDatePos - 2];

                        lpszDate[cubDateLen] = EOS;

                        /*----------------------------------------------------*
                         * Ueberpruefen, welches Jahrhundert einzublenden ist *
                         *----------------------------------------------------*/
                        ubDummy1 = (UBYTE) (ubYear[2] * 10 + ubYear[3]);
                        ubDummy2 = (UBYTE) ( (lpszDate[cub + 1] - '0') * 10
                                            + lpszDate[cub + 2] - '0');

                        if (ubDummy1 < ubDummy2)
                        {                   /* Voriges Jahrhundert einblenden */
                           lpszDate[cub - 1] = (char) ('0' + ubPreY[0]);   
                           lpszDate[cub] = (char) ('0' + ubPreY[1]);
                        }
                        else                 /* Dieses Jahrhundert einblenden */
                        {
                           lpszDate[cub - 1] = (char) ('0' + ubYear[0]);
                           lpszDate[cub] = (char) ('0' + ubYear[1]);
                        }
                        cub += 2;
                        ubFormatPos += 2;
                     }            /* Ende if (Jahreszahl von 2 auf 4 Ziffern) */
                     break;                       /* Von case 2. Jahresziffer */
                     
                  /*----------------------------------------------------------*
                   * 3. Jahresziffer pruefen                                  *
                   *----------------------------------------------------------*/
                  case 3:

                     if (ubCharIs != CHARIS_DIGIT)
                     {       /* ...dann Jahreszahl auf 4 Ziffern expandieren! */
                        if (cubDateLen < cubFormatLen) cubDateLen++;

                        /* Ab aktuelle Position alles um 1 Stelle nach rechts */
                        for(ubDatePos = cubDateLen; ubDatePos > cub;ubDatePos--)
                           lpszDate[ubDatePos] = lpszDate[ubDatePos - 1];

                        /* Jahresziffern 1 + 2 auf Plaetze 3 bzw. 4 kopieren  */
                        lpszDate[cub] = lpszDate[cub - 2];
                        lpszDate[cub + 1] = lpszDate[cub - 1];
                        lpszDate[cubDateLen] = EOS;

                        /*----------------------------------------------------*
                         * Ueberpruefen, welches Jahrhundert einzublenden ist *
                         *----------------------------------------------------*/
                        ubDummy1 = (UBYTE) (ubYear[2] * 10 + ubYear[3]);
                        ubDummy2 = (UBYTE) ( (lpszDate[cub] - '0') * 10
                                            + lpszDate[cub + 1] - '0');

                        if (ubDummy1 < ubDummy2)
                        {                  /* Voriges Jahrhundert einblenden? */
                           lpszDate[cub - 2] = (char) ('0' + ubPreY[0]);
                           lpszDate[cub - 1] = (char) ('0' + ubPreY[1]);
                        }
                        else                 /* Dieses Jahrhundert einblenden */
                        {
                           lpszDate[cub - 2] = (char) ('0' + ubYear[0]);
                           lpszDate[cub - 1] = (char) ('0' + ubYear[1]);
                        }
                        cub++;
                        ubFormatPos++;
                     }                               /* Ende if (ubCharIs...) */
                     break;                       /* Von case 3. Jahresziffer */
                     
                  /*----------------------------------------------------------*
                   * 4. Jahresziffer pruefen                                  *
                   *----------------------------------------------------------*/
                  case 4:

                     if (ubCharIs != CHARIS_DIGIT)
                     {
                        lpszDate[cub] = 'A';
                        ubFormatPos--;
                        cub--;
                        break;           /* Im naechsten Durchgang raus damit */
                     }
                     break;                       /* Von case 4. Jahresziffer */

                  default: /* Mehr als 4 Jahresziffern gibt's doch gar nicht! */
                     lpszDate[0] = EOS;
                     return( FALSE );          /* Kein korrekter Formatstring */
               }                                 /* Ende switch (ubFormatPos) */
               break;                                        /* Von case Jahr */

            /*----------------------------------------------------------------*
             * Monat bearbeiten                                               *
             *----------------------------------------------------------------*/
            case 'M':
            case 'm':
               /* Merker ueber die Format-String-Analyse aktualisieren        */
               ubFormatPos++;

               if (chLastBez != 'M')    /* Kommt Monatsformat zum 1. Mal vor? */
               {
                  chLastBez = 'M';
                  cubParams++;
                  ubFormatPos = 1;
               }
               switch (ubFormatPos)    /* Monatsziffer Pos.abhaengig pruefen */
               {
                  /*----------------------------------------------------------*
                   * 1. Monatsziffer pruefen                                  *
                   *----------------------------------------------------------*/
                  case 1:

                     if (ubCharIs == CHARIS_FORMT) /* Format an falscher Pos.?*/
                     {
                        lpszDate[cub] = 'A';
                        ubFormatPos--;
                        cub--;
                        break;           /* Im naechsten Durchgang raus damit */
                     }
                     if (ubCharIs == CHARIS_SPACE)/* ' ' als 0 interpretieren */
                        lpszDate[cub] = '0';

                     if (ubCharIs == CHARIS_DIGIT && lpszDate[cub] > '1')
                     {                      /* Eine Ziffer schon ausreichend? */
                        if (cubDateLen < cubFormatLen) cubDateLen++;                        

                        /* Ab aktuelle Position alles um 1 Stelle nach rechts */
                        for(ubDatePos = cubDateLen; ubDatePos > cub;ubDatePos--)
                           lpszDate[ubDatePos] = lpszDate[ubDatePos - 1];

                        /* 1. Monatsziffer auf Position 2 kopieren            */
                        lpszDate[cub + 1] = lpszDate[cub];
                        lpszDate[cub] = '0';        /* 1.Ziffer = Fuehrende 0 */
                        lpszDate[cubDateLen] = EOS;
                        cub++;
                        ubFormatPos++;
                     } 
                     break;                       /* Von case 1. Monatsziffer */

                  /*----------------------------------------------------------*
                   * 2. Monatsziffer pruefen                                  *
                   *----------------------------------------------------------*/
                  case 2:

                     if (   (   ubCharIs != CHARIS_DIGIT      /* Keine Ziffer */
                             || lpszDate[cub] == '0'    )  /* oder Ziffer = 0 */
                         && (   lpszDate[cub - 1] == '0'      /* und Monat 0? */
                             && !fIgnoreZero           ))/* obwohl unerlaubt? */
                     {
                        lpszDate[cub] = 'A';
                        ubFormatPos--;
                        cub--;
                        break;           /* Im naechsten Durchgang raus damit */
                     }
                     if (   ubCharIs != CHARIS_DIGIT
                         || (lpszDate[cub - 1] == '1' && lpszDate[cub] > '2') )
                     {    /* Trennzeichen oder eine Ziffer schon ausreichend? */
                        if (cubDateLen < cubFormatLen) cubDateLen++;                        

                        /* Ab aktuelle Position alles um 1 Stelle nach rechts */
                        for(ubDatePos = cubDateLen; ubDatePos > cub;ubDatePos--)
                           lpszDate[ubDatePos] = lpszDate[ubDatePos - 1];

                        lpszDate[cubDateLen] = EOS;
                        /* 1. Monatsziffer auf Position 2 kopieren            */
                        lpszDate[cub] = lpszDate[cub - 1];
                        lpszDate[cub - 1] = '0';   /* 1. Ziffer = Fuehrende 0 */
                     } 
                     break;                       /* Von case 2. Monatsziffer */

                  default:  /* Mehr als 2 Monatsziffern gibts doch gar nicht! */
                     lpszDate[0] = EOS;
                     return ( FALSE );        /* Kein korrekter Format-String */
               }                                 /* Ende switch (ubFormatPos) */
               break;                                       /* Von case Monat */

            /*----------------------------------------------------------------*
             * Tag bearbeiten                                                 *
             *----------------------------------------------------------------*/
            case 'D':
            case 'd':
               /* Merker ueber die Format-String-Analyse aktualisieren        */
               ubFormatPos++;

               if (chLastBez != 'D')     /* Kommt Tagesformat zum 1. Mal vor? */
               {
                  chLastBez = 'D';
                  cubParams++;
                  ubFormatPos = 1;
               }
               switch (ubFormatPos)      /* Tagesziffer Pos.abhaengig pruefen */
               {
                  /*----------------------------------------------------------*
                   * 1. Tagesziffer pruefen                                   *
                   *----------------------------------------------------------*/
                  case 1:

                     if (ubCharIs == CHARIS_FORMT) /* Format an falscher Pos.?*/
                     {
                        lpszDate[cub] = 'A';
                        ubFormatPos--;
                        cub--;
                        break;           /* Im naechsten Durchgang raus damit */
                     } 
                     if (ubCharIs == CHARIS_SPACE)/* ' ' als 0 interpretieren */
                        lpszDate[cub] = '0';

                     if (ubCharIs == CHARIS_DIGIT && lpszDate[cub] > '3')
                     {                      /* Eine Ziffer schon ausreichend? */
                        if (cubDateLen < cubFormatLen) cubDateLen++;

                        /* Ab aktuelle Position alles um 1 Stelle nach rechts */
                        for(ubDatePos = cubDateLen; ubDatePos > cub;ubDatePos--)
                           lpszDate[ubDatePos] = lpszDate[ubDatePos - 1];

                        /* 1. Tagesziffer auf Position 2 kopieren             */
                        lpszDate[cub + 1] = lpszDate[cub];
                        lpszDate[cub] = '0';       /* 1. Ziffer = Fuehrende 0 */
                        lpszDate[cubDateLen] = EOS;
                        cub++;
                        ubFormatPos++;
                     } 
                     break;                        /* Von case 1. Tagesziffer */

                  /*----------------------------------------------------------*
                   * 2. Tagesziffer pruefen                                   *
                   *----------------------------------------------------------*/
                  case 2:  

                     if (   (   ubCharIs != CHARIS_DIGIT      /* Keine Ziffer */
                             || lpszDate[cub] == '0'    )           /* oder 0 */
                         && (   lpszDate[cub - 1] == '0'      /* und Monat 0? */
                             && !fIgnoreZero           ))/* obwohl unerlaubt? */
                     {
                        lpszDate[cub] = 'A';
                        ubFormatPos--;
                        cub--;
                        break;           /* Im naechsten Durchgang raus damit */
                     } 
                     if (   ubCharIs != CHARIS_DIGIT
                         || (lpszDate[cub - 1] == '3' && lpszDate[cub] > '1' ) )
                     {    /* Trennzeichen oder eine Ziffer schon ausreichend? */
                        if (cubDateLen < cubFormatLen) cubDateLen++;                        

                        /* Ab aktuelle Position alles um 1 Stelle nach rechts */
                        for(ubDatePos = cubDateLen; ubDatePos > cub;ubDatePos--)
                           lpszDate[ubDatePos] = lpszDate[ubDatePos - 1];

                        lpszDate[cubDateLen] = EOS;
                        /* 1. Tagesziffer auf Position 2 kopieren             */
                        lpszDate[cub] = lpszDate[cub - 1];
                        lpszDate[cub - 1] = '0';   /* 1. Ziffer = Fuehrende 0 */
                     } 
                     break;                        /* Von case 2. Tagesziffer */

                  default:   /* Mehr als 2 Tagesziffern gibts doch gar nicht! */
                     lpszDate[0] = EOS;
                     return ( FALSE );        /* Kein korrekter Format-String */
               }                                 /* Ende switch (ubFormatPos) */
               break;                                         /* Von case Tag */

            /*----------------------------------------------------------------*
             * Weder Y, M, D gefunden => Format-Zeichen in Ausgabe kopieren   *
             *----------------------------------------------------------------*/
            default:
               /* Merker ueber die Format-String-Analyse aktualisieren        */
               chLastBez = ' ';   

               if (ubFormatPos < 2 || (ubFormatPos > 2 && ubFormatPos < 4) )
               {
                  lpszDate[0] = EOS;
                  return( FALSE );             /* kein korrekter Formatstring */
               }
               if (CheckDateFormat( szFormat[cub] ) )  /* Hier Formatzeichen? */
               {
                  if (cub < cubDateLen)
                  {
                     /*-------------------------------------------------------*
                      * Muss ein Format-Zeichen eingefuegt werden?            *
                      *-------------------------------------------------------*/
                     if (ubCharIs == CHARIS_DIGIT)
                     {
                        if (cubDateLen < cubFormatLen) cubDateLen++;                        

                        /* Ab aktueller Position alles um 1 Stelle nach rechts*/                
                        for(ubDatePos = cubDateLen; ubDatePos > cub;ubDatePos--)
                           lpszDate[ubDatePos] = lpszDate[ubDatePos - 1];

                        lpszDate[cubDateLen] = EOS;
                     } 
                     lpszDate[cub] = szFormat[cub];   /* Formatzeichen setzen */
                  }
               }
               else                                     /* Kein Formatzeichen */
               {
                  lpszDate[0] = EOS;
                  return( FALSE );               /* Ungueltiges Formatzeichen */
               }
               break;                                          /* Von default */
         }                                        /* Ende switch (szFormat[]) */
         cub++;
      }                                        /* Ende else Gueltiges Zeichen */
   }                                                  /* Ende Auswertschleife */
   if (cubParams > 3)                             /* Zuviele Formatparameter? */
   {
      lpszDate[0] = EOS;
      return( FALSE );                        /* Kein korrekter Format-String */
   }
   cubDateLen = (UBYTE) GetDlgItemText( hDlg, uwIdCtrl, szd, 11 );
   if (!cubDateLen) return( FALSE );

   if (!lstrcmp( szd, lpszDate ) ) return( FALSE );

   return( TRUE ); 
}                                                 /* Ende GetDlgItemDateStr() */

/******************************************************************************/
/*                                                                            */
/* Funktionsname: StrToTime                                                   */
/*                                                                            */
/* Beschreibung : String nach Uhrzeit konvertieren                            */
/*                                                                            */
/* Autor        : Michael G. Wetzel            Abt.: Siemens AG UB Med DRE    */
/* -------------------------------------------------------------------------- */
/* Parameter    : lpubHour   = Puffer fuer die Stunde                         */
/*                lpubMinute = Puffer fuer die Minute                         */
/*                lpubSecond = Puffer fuer die Sekunde                        */
/*                lpszTime   = Puffer mit Uhrzeit-String                      */
/*                lpszFormat = Format-String, H,M,S als Bezeichner,           */
/*                             z.B. "HH:MM:SS"                                */
/*                           = NULL => Internes landesspezifisches Format     */
/*                                                                            */
/* Rueckgabewert: TRUE wenn OK sonst FALSE                                    */
/*                                                                            */
/* Externdaten  : Name                         Zugriff (r/w/&)                */
/*                hDLLInstGl                   r                              */
/*                                                                            */
/* Bemerkungen  : Die Rueckgabepuffer sind im Fehlerfall mit 0 initialisiert. */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.1_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 18.02.93 -    Entwurf                                                      */
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/* 03.12.93 -    szPFORMAT von Laenge 5 auf 3 (SK)                            */
/* 13.03.95 -    + IDS_TIMEFORMAT (Dietmar Keller) => V.1.1                   */
/* 14.03.95 -    Review letzte Aenderung (Juergen Zimmermann)                 */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI StrToTime( LPUBYTE lpubHour, LPUBYTE lpubMinute, LPUBYTE lpubSecond,
                       LPCSTR lpszTime, LPCSTR lpszFormat )
{
          UBYTE ubTimePos    = 0;  /* Position des aktuellen Uhrzeit-Zeichens */
          UBYTE ubFormatPos  = 0;  /* Position des aktuellen Format-Zeichens  */
          UBYTE ubParamLen   = 0;  /* Laenge des aktuellen Format-Parameters  */
          UBYTE ubParamCount = 0;  /* Zaehler fuer gefundene Format-Parameter */
          UBYTE ubHour       = 0;  /* Lokale Stunde                           */
          UBYTE ubMinute     = 0;  /* Lokale Minute                           */
          UBYTE ubSecond     = 0;  /* Lokale Sekunde                          */
          char  szFormat[FORMATSTRLEN]; /* Zeit-Formatstring                  */
   static char  szPFORMAT[3];   /* Puffer fuer Format; static wg.atoi() & DLL */

   *lpubHour   = 0;
   *lpubMinute = 0;
   *lpubSecond = 0;

   /*-------------------------------------------------------------------------*
    * Evtl. Default-Format benutzen                                           *
    *-------------------------------------------------------------------------*/
   if (!lpszFormat)
   {
       return( FALSE );
   }
   else
      (void) lstrcpy( szFormat, lpszFormat );
      
   while (   szFormat[ubFormatPos] != EOS 
          && lpszTime  [ubTimePos] != EOS 
          && ubParamCount           < 3  )
   {
      switch (szFormat[ubFormatPos])
      {
         /*-------------------------------------------------------------------*/
         /* Stunde bearbeiten                                                 */
         /*-------------------------------------------------------------------*/
         case 'H':
         case 'h':
            if (ubHour) return( FALSE );           /* Ueberzaehlige 'H'       */
            
            for ( ubParamLen = 0;
                     toupper( szFormat[ubFormatPos] ) == 'H'
                  && isdigit( lpszTime[ubTimePos] );
                  ubParamLen++, ubFormatPos++, ubTimePos++)
               ;

            if (ubParamLen == 2)                   /* 2-stellige Angabe       */
            {
               szPFORMAT[0] = lpszTime[ubTimePos - 2];
               szPFORMAT[1] = lpszTime[ubTimePos - 1];
               szPFORMAT[2] = EOS;
            }
            else
            {
               if (ubParamLen == 1)                /* 1-stellige Angabe       */
               {
                  szPFORMAT[0] = lpszTime[ubTimePos - 1];
                  szPFORMAT[1] = EOS;
                  if (toupper( szFormat[ubFormatPos] ) == 'H') ubFormatPos++;
               }
               else
                  return( FALSE );                 /* Nicht 2- oder 1-stellig */
            }
            ubHour = (UBYTE) atoi( szPFORMAT );
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Minute bearbeiten                                                 */
         /*-------------------------------------------------------------------*/
         case 'M':
         case 'm':
            if (ubMinute) return( FALSE );         /* Ueberzaehlige 'M'       */
            
            for ( ubParamLen = 0;
                     toupper( szFormat[ubFormatPos] ) == 'M'
                  && isdigit( lpszTime[ubTimePos] );
                  ubParamLen++, ubFormatPos++, ubTimePos++)
               ;

            if (ubParamLen == 2)                      /* 2-stellige Angabe    */
            {
               szPFORMAT[0] = lpszTime[ubTimePos - 2];
               szPFORMAT[1] = lpszTime[ubTimePos - 1];
               szPFORMAT[2] = EOS;
            }
            else
            {
               if (ubParamLen == 1)                   /* 1-stellige Angabe    */
               {
                  szPFORMAT[0] = lpszTime[ubTimePos - 1];
                  szPFORMAT[1] = EOS;
                  if (toupper( szFormat[ubFormatPos] ) == 'M') ubFormatPos++;
               }
               else
                  return( FALSE );                 /* Nicht 1- oder 2-stellig */
            }
            ubMinute = (UBYTE) atoi( szPFORMAT );
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Sekunde bearbeiten                                                */
         /*-------------------------------------------------------------------*/
         case 'S':
         case 's':
            if (ubSecond) return( FALSE );         /* Ueberzaehlige 'S'       */
            
            for ( ubParamLen = 0;
                     toupper( szFormat[ubFormatPos]) == 'S'
                  && isdigit( lpszTime[ubTimePos]);
                  ubParamLen++, ubFormatPos++, ubTimePos++)
               ;

            if (ubParamLen == 2)                  /* 2-stellige Angabe        */
            {
               szPFORMAT[0] = lpszTime[ubTimePos - 2];
               szPFORMAT[1] = lpszTime[ubTimePos - 1];
               szPFORMAT[2] = EOS;
            }
            else
            {
               if (ubParamLen == 1)               /* 1-stellige Angabe        */
               {
                  szPFORMAT[0] = lpszTime[ubTimePos - 1];
                  szPFORMAT[1] = EOS;
                  if (toupper( szFormat[ubFormatPos] ) == 'S') ubFormatPos++;
               }
               else
                  return( FALSE);                 /* Nicht 1- oder 2-stellig  */
            }
            ubSecond = (UBYTE) atoi( szPFORMAT );
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*/
         /* Weder H, M, S gefunden                                            */
         /*-------------------------------------------------------------------*/
         default:
            ubFormatPos++;

            if (CheckTimeFormat( lpszTime[ubTimePos] ) )
               ubTimePos++;
            else
               return ( FALSE );

            break;
      }
   }
   /*-------------------------------------------------------------------------*
    * Alles ok => Lokale Werte uebernehmen                                    *
    *-------------------------------------------------------------------------*/
   *lpubHour   = ubHour;
   *lpubMinute = ubMinute;
   *lpubSecond = ubSecond;
   return( TRUE );
}                                                         /* Ende StrToTime() */

/******************************************************************************/
/*                                                                            */
/* Funktionsname: StrToDate                                                   */
/*                                                                            */
/* Beschreibung : String nach Datum konvertieren                              */
/*                                                                            */
/* Autor        : MGW/Hanmen                   Abt.: Siemens AG UB Med DRE    */
/* -------------------------------------------------------------------------- */
/* Parameter    : lpuwYear   = Jahrpuffer                                     */
/*                lpubMonth  = Monatspuffer                                   */
/*                lpubDay    = Tagpuffer                                      */
/*                lpszDate   = Puffer mit Datums-String                       */
/*                lpszFormat = Format-String (Y,M,D als Bezeichner)           */
/*                           = NULL => Internes landesspezifisches Format     */
/*                                                                            */
/* Rueckgabewert: TRUE wenn OK, sonst FALSE.                                  */
/*                                                                            */
/* Externdaten  : Name                         Zugriff (r/w/&)                */
/*                hDLLInstGl                   r                              */
/*                                                                            */
/* Bemerkungen  : Jahreszahl wird nur 2- oder 4-stellig akzeptiert.           */
/*                Die Puffer werden im Fehlerfall mit den jeweiligen Maximal- */
/*                werten initialisiert.                                       */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version _1.1_                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 19.10.92 -    Funktion fuer Klein- und Grossbuchstaben im Format (MGW)     */
/* 19.10.92 -    Kuerzere Ziffern-Angaben als im Formatstring zugelassen (MGW)*/
/* 27.10.93 -    Review zur NULL - Serie     (K. Schrick   )                  */
/* 08.11.93 -    *lpuwYear groesser 0 und kleiner 100, dann += 1900           */
/* 27.06.94 -    Fehlerinitialisierung jetzt mit Maximalwerten (J.Zimmermann) */
/* 13.03.95 -    + IDS_DATEFORMAT (Dietmar Keller) => V.1.1                   */
/* 14.03.95 -    Review letzte Aenderung (Juergen Zimmermann)                 */
/*                                                                            */
/******************************************************************************/
BOOL WINAPI StrToDate( LPUWORD lpuwYear, LPUBYTE lpubMonth, LPUBYTE lpubDay,
                       LPCSTR lpszDate, LPCSTR lpszFormat )
{
          UBYTE ubDatePos    = 0;  /* Position des aktuellen Datums-Zeichens  */
          UBYTE ubFormatPos  = 0;  /* Position des aktuellen Format-Zeichens  */
          UBYTE ubParamLen   = 0;  /* Laenge des aktuellen Format-Parameters  */
          UBYTE ubParamCount = 0;  /* Zaehler fuer gefundene Format-Parameter */
          UWORD uwYear       = 0;  /* Lokales Jahr                            */
          UBYTE ubMonth      = 0;  /* Lokaler Monat                           */
          UBYTE ubDay        = 0;  /* Lokaler Tag                             */
          char  szFormat[FORMATSTRLEN]; /* Datums-Formatstring                */
   static char  szPFORMAT[5];   /* Puffer fuer Format; static wg.atoi() & DLL */

   *lpuwYear  = 0;
   *lpubMonth = 0;
   *lpubDay   = 0;

   /*-------------------------------------------------------------------------*
    * Evtl. Default-Format benutzen                                           *
    *-------------------------------------------------------------------------*/
   if (!lpszFormat)
   {
       return( FALSE );
   }
   else
      (void) lstrcpy( szFormat, lpszFormat );
      
   while (   szFormat[ubFormatPos] != EOS
          && lpszDate[ubDatePos  ] != EOS 
          && ubParamCount           < 3  )
   {
      switch (szFormat[ubFormatPos])
      {
         /*-------------------------------------------------------------------*
          * Jahr bearbeiten                                                   *
          *-------------------------------------------------------------------*/
         case 'Y':
         case 'y':
            if (uwYear) return( FALSE );           /* Ueberzaehlige 'Y'       */
            
            for ( ubParamLen = 0;
                     toupper( szFormat[ubFormatPos] ) == 'Y' 
                  && isdigit( lpszDate[ubDatePos] );
                  ubParamLen++, ubFormatPos++, ubDatePos++)
               ;

            if (ubParamLen == 4)                   /* 4-stellige Angabe       */
            {
               szPFORMAT[0] = lpszDate[ubDatePos - 4];
               szPFORMAT[1] = lpszDate[ubDatePos - 3];
               szPFORMAT[2] = lpszDate[ubDatePos - 2];
               szPFORMAT[3] = lpszDate[ubDatePos - 1];
               szPFORMAT[4] = EOS;
            }
            else
            {
               if (ubParamLen == 2)                /* 2-stellige Angabe       */
               {
                  szPFORMAT[0] = lpszDate[ubDatePos - 2];
                  szPFORMAT[1] = lpszDate[ubDatePos - 1];
                  szPFORMAT[2] = EOS;
                  if (toupper( szFormat[ubFormatPos] ) == 'Y') ubFormatPos++;
                  if (toupper( szFormat[ubFormatPos] ) == 'Y') ubFormatPos++;
               }
               else
               {
                  if (ubParamLen == 1)             /* 1-stellige Angabe       */
                  {
                     szPFORMAT[0] = lpszDate[ubDatePos - 1];
                     szPFORMAT[1] = EOS;

                     while (toupper( szFormat[ubFormatPos] ) == 'Y') 
                        ubFormatPos++;
                  }
                  else
                     return( FALSE );          /* Nicht 4-, 2- oder 1-stellig */
               }
            }
            uwYear = (UWORD) atoi( szPFORMAT );
            if (uwYear && uwYear < 100) uwYear += 1900;
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*
          * Monat bearbeiten                                                  *
          *-------------------------------------------------------------------*/
         case 'M':
         case 'm':
            if (ubMonth) return( FALSE );          /* Ueberzaehlige 'M'       */
            
            for ( ubParamLen = 0;
                     toupper( szFormat[ubFormatPos] ) == 'M'
                  && isdigit( lpszDate[ubDatePos] );
                  ubParamLen++, ubFormatPos++, ubDatePos++)
               ;

            if (ubParamLen == 1)                   /* 1-stellig Angabe        */
            {
               szPFORMAT[0] = lpszDate[ubDatePos - 1];
               szPFORMAT[1] = EOS;
               if (toupper( szFormat[ubFormatPos] ) == 'M') ubFormatPos++;
            }
            else
            {
               if (ubParamLen == 2)                /* 2-stellige Angabe       */
               {
                  szPFORMAT[0] = lpszDate[ubDatePos - 2];
                  szPFORMAT[1] = lpszDate[ubDatePos - 1];
                  szPFORMAT[2] = EOS;
               }
               else
                  return( FALSE );                 /* Nicht 1- oder 2-stellig */
            }
            ubMonth = (UBYTE) atoi( szPFORMAT );
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*
          * Tag bearbeiten                                                    *
          *-------------------------------------------------------------------*/
         case 'D':
         case 'd':
            if (ubDay) return( FALSE );            /* Ueberzaehlige 'D'       */
            
            for (ubParamLen = 0;
                    toupper( szFormat[ubFormatPos] ) == 'D' 
                 && isdigit( lpszDate[ubDatePos] );
                 ubParamLen++, ubFormatPos++, ubDatePos++)
               ;

            if (ubParamLen == 1)                  /* 1-stellige Angabe        */
            {
               szPFORMAT[0] = lpszDate[ubDatePos - 1];
               szPFORMAT[1] = EOS;
               if (toupper( szFormat[ubFormatPos] ) == 'D') ubFormatPos++;
            }
            else
            {
               if (ubParamLen == 2)               /* 2-stellige Angabe        */
               {
                  szPFORMAT[0] = lpszDate[ubDatePos - 2];
                  szPFORMAT[1] = lpszDate[ubDatePos - 1];
                  szPFORMAT[2] = EOS;
               }
               else
                  return( FALSE );                /* Nicht 1- oder 2-stellig  */
            }
            ubDay = (UBYTE) atoi( szPFORMAT );
            ubParamCount++;
            break;

         /*-------------------------------------------------------------------*
          * Weder Y, M, D gefunden                                            *
          *-------------------------------------------------------------------*/
         default:
            ubFormatPos++;

            if (CheckDateFormat( lpszDate[ubDatePos] ) )
               ubDatePos++;
            else
               return( FALSE );
               
            break;
      }
   }                                              
   /*-------------------------------------------------------------------------*
    * Alles ok => Lokale Werte uebernehmen                                    *
    *-------------------------------------------------------------------------*/
   *lpuwYear  = uwYear;
   *lpubMonth = ubMonth;
   *lpubDay   = ubDay;
   return( TRUE );
}                                                         /* Ende StrToDate() */

/******************************************************************************/
/*                                                                            */
/* Funktionsname: CheckTimeFormat                                             */
/*                                                                            */
/* Beschreibung : Formatzeichen fuer die Zeit auf Gueltigkeit ueberpruefen    */
/*                                                                            */
/* Autor        : K. Schrick                   Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : chTimeFormat = Zeichen, das zum Formatieren der Zeit dient  */
/*                                                                            */
/* Rueckgabewert: TRUE wenn OK sonst FALSE                                    */
/*                                                                            */
/* Externdaten  : Name                         Zugriff (r/w)                  */
/*                szTimeFormatGl[]             r                              */
/*                                                                            */
/* Bemerkungen  : Zur Ueberpruefung dient ein vordefinierter String           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version 01.00                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 06.12.93 -    Entwurf                                                      */
/*                                                                            */
/******************************************************************************/
static BOOL CheckTimeFormat ( char chTimeFormat )
{
   LPCSTR lpszFormat;
  
   lpszFormat = szTimeFormatGl;
   
   while (*lpszFormat != EOS)
   {
      if (chTimeFormat == *lpszFormat) return( TRUE );

      lpszFormat++;
   }
   return( FALSE );
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: CheckDateFormat                                             */
/*                                                                            */
/* Beschreibung : Formatzeichen fuer das Datum auf Gueltigkeit ueberpruefen   */
/*                                                                            */
/* Autor        : K. Schrick                   Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : chDateFormat = Zeichen, das z. Formatieren des Datums dient */
/*                                                                            */
/* Rueckgabewert: TRUE wenn OK sonst FALSE                                    */
/*                                                                            */
/* Externdaten  : Name                         Zugriff (r/w)                  */
/*                szDateFormatGl[]             r                              */
/*                                                                            */
/* Bemerkungen  : Zur Ueberpruefung dient ein vordefinierter String           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version 01.00                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 06.12.93 -    Entwurf                                                      */
/*                                                                            */
/******************************************************************************/
static BOOL CheckDateFormat ( char chDateFormat )
{
   LPCSTR lpszFormat;
  
   lpszFormat = szDateFormatGl;
   
   while (*lpszFormat != EOS)
   {
      if (chDateFormat == *lpszFormat) return( TRUE );

      lpszFormat++;
   }
   return( FALSE );
}

/******************************************************************************/
/*                                                                            */
/* Funktionsname: IsLeapYear                                                  */
/*                                                                            */
/* Beschreibung : Testet ob Jahr ein Schaltjahr ist                           */
/*                                                                            */
/* Autor        : M. Wetzel (MGW)              Abt.: Siemens AG UB Med DER 32 */
/* -------------------------------------------------------------------------- */
/* Parameter    : uwYear - Zu testendes Jahr                                  */
/*                                                                            */
/* Rueckgabewert: TRUE wenn Schaltjahr, sonst FALSE                           */
/*                                                                            */
/* Externdaten  : -                                                           */
/*                                                                            */
/* Bemerkungen  : -                                                           */
/*                                                                            */
/* Sicherheit   : -                                                           */
/* -------------------------------------------------------------------------- */
/* Status       : Entwurf                                                     */
/* -------------------------------------------------------------------------- */
/* Aenderungshistorie - Version 01.00                                         */
/*                                                                            */
/* Datum    -    Modifikation                                                 */
/* 25.08.94 -    Version 1.0 (MGW)                                            */
/*                                                                            */
/******************************************************************************/
static BOOL IsLeapYear( UWORD uwYear )
{
    return( (!(uwYear % 4) && (uwYear % 100)) || !(uwYear % 400) );
}
