﻿// <copyright file="App.xaml.cs" company="Sirona Dental Systems GmbH">
//    Copyright (c) Sirona Dental Systems GmbH. All rights reserved.
// </copyright>

namespace PTest
{
    using System;
    using System.IO;
    using System.Reflection;
    using System.Windows;
    using NGOptMan;
    using Properties;

    public partial class App : Application
    {
        public static ApplicationData appData;

        private readonly string _appPath; // Application path with trailing backslash

        public string sidexisFile;
        public string slidaFile;

        private App()
        {
            slidaFile   = Settings.Default.SlidaPath;
            sidexisFile = Settings.Default.SidexisPath;

            try // Try to find SIDIIN in SIDEXIS installation directory
            {
                var optMan = (IOptionsManager) Activator.CreateInstance( Type.GetTypeFromProgID( "SIDEXISNG.OptionsManager" ) );

                if (sidexisFile == string.Empty)
                {
                    sidexisFile = optMan.GetOption( "SidexisPath", "" ); // Always includes trailing backslash
                    sidexisFile = Path.Combine( sidexisFile, "Sidexis.exe" );
                }
                if (slidaFile == string.Empty)
                    slidaFile = optMan.GetOption( "Sifiledb.ini/FromStation0/File", "" );
            }
            catch (Exception) // No OptMan?
            {
                // Do nothing
            }
            _appPath = Assembly.GetExecutingAssembly().Location;
            _appPath = GetDirWithTrailingBackslash( Path.GetDirectoryName( _appPath ) );

            appData = new ApplicationData();
        }
        
        public PTestModel PTestData
        {
            get
            {
                return appData.Model;
            }
        }

        public void Save()
        {
            ApplicationData.Save( appData, _appPath );
        }

        public void Load()
        {
            ApplicationData.Load( out appData, _appPath );
        }

        private static string GetDirWithTrailingBackslash( string directory )
        {
            if (directory.Substring( directory.Length - 1, 1 ) != "\\")
                directory += "\\";
            return directory;
        }

        protected override void OnExit(ExitEventArgs e)
        {
            Settings.Default.SlidaPath   = slidaFile;
            Settings.Default.SidexisPath = sidexisFile;

            Settings.Default.Save();

            base.OnExit( e );
        }
    }
}
