﻿// <copyright file="ApplicationData.cs" company="Sirona Dental Systems GmbH">
//    Copyright (c) Sirona Dental Systems GmbH. All rights reserved.
// </copyright>

namespace PTest
{
    using System;
    using System.IO;
    using System.Windows;
    using System.Xml.Serialization;
    using Properties;

    public class ApplicationData
    {
        public PTestModel Model { get; set; }

        private const string XmlFileName = @"ApplicationData.xml";

        public ApplicationData()
        {
            Model = new PTestModel();
        }

        public static void Load( out ApplicationData appData, string strDataPath )
        {
            try
            {
                var settingsFile = strDataPath + XmlFileName;
                var fiSet        = new FileInfo( settingsFile );

                if (!fiSet.Exists) // Settings not yet there?
                {
                    var gpdNew = new ApplicationData();

                    Save( gpdNew, strDataPath ); // Then create it
                }
                var fileStream = new FileStream( settingsFile, FileMode.Open, FileAccess.Read );
                var ser        = new XmlSerializer( typeof( ApplicationData ) );

                appData = (ApplicationData) ser.Deserialize( fileStream );
                fileStream.Close();
            }
            catch (Exception ex)
            {
                appData = null;
                MessageBoxEx.Show( Resources.ApplicationName, Resources.ApplicationData_ErrorLoad + "\n" + ex.Message, MessageBoxButton.OK, MessageBoxExImage.Error );
            }
        }
        
        public static void Save( ApplicationData appData, string strDataPath )
        {
            try
            {
                var settingsFile = strDataPath + XmlFileName;
                var fileStream   = new FileStream( settingsFile, FileMode.Create, FileAccess.Write );
                var ser          = new XmlSerializer( typeof( ApplicationData ) );

                ser.Serialize( fileStream, appData );

                fileStream.Flush();
                fileStream.Close();
            }
            catch (Exception ex)
            {
                MessageBoxEx.Show( Resources.ApplicationName, Resources.ApplicationData_ErrorSave + "\n" + ex.Message, MessageBoxButton.OK, MessageBoxExImage.Error );
            }
        }
    }
}
