﻿// <copyright file="MessageBoxEx.xaml.cs" company="Sirona Dental Systems GmbH">
//    Copyright (c) Sirona Dental Systems GmbH. All rights reserved.
// </copyright>


using System;
using System.Windows;
using System.Windows.Media.Imaging;

namespace PTest
{
    using System.Linq;
    using Microsoft.SqlServer.Server;

    public enum MessageBoxExType
    {
        ConfirmationWithYesNo = 0,
        ConfirmationWithYesNoCancel,
        Information,
        Error,
        Warning
    }

    public enum MessageBoxExImage
    {
        Warning = 0,
        Question,
        Information,
        Error,
        None
    }

    public partial class MessageBoxEx : Window
    {
        private MessageBoxEx()
        {
            InitializeComponent();
        }
        static MessageBoxEx _messageBox;
        static MessageBoxResult _result = MessageBoxResult.No;
        public static MessageBoxResult Show
        (string caption, string msg, MessageBoxExType type)
        {
            switch (type)
            {
                case MessageBoxExType.ConfirmationWithYesNo:
                    return Show(caption, msg, MessageBoxButton.YesNo,
                    MessageBoxExImage.Question);
                case MessageBoxExType.ConfirmationWithYesNoCancel:
                    return Show(caption, msg, MessageBoxButton.YesNoCancel,
                    MessageBoxExImage.Question);
                case MessageBoxExType.Information:
                    return Show(caption, msg, MessageBoxButton.OK,
                    MessageBoxExImage.Information);
                case MessageBoxExType.Error:
                    return Show(caption, msg, MessageBoxButton.OK,
                    MessageBoxExImage.Error);
                case MessageBoxExType.Warning:
                    return Show(caption, msg, MessageBoxButton.OK,
                    MessageBoxExImage.Warning);
                default:
                    return MessageBoxResult.No;
            }
        }
        public static MessageBoxResult Show(string msg, MessageBoxExType type)
        {
            return Show(string.Empty, msg, type);
        }
        public static MessageBoxResult Show(string msg)
        {
            return Show(string.Empty, msg, MessageBoxButton.OK, MessageBoxExImage.None);
        }
        public static MessageBoxResult Show
        (string caption, string text)
        {
            return Show(caption, text, MessageBoxButton.OK, MessageBoxExImage.None);
        }
        public static MessageBoxResult Show
        (string caption, string text, MessageBoxButton button)
        {
            return Show(caption, text, button,MessageBoxExImage.None);
        }
        public static MessageBoxResult Show(string caption, string text, MessageBoxButton button, MessageBoxExImage image)
        {
            _messageBox = new MessageBoxEx{ txtMsg = { Text = text }};

            _messageBox.Title = caption;

            SetVisibilityOfButtons(button);
            SetImageOfMessageBox(image);

            var owner = Application.Current.MainWindow;

            _messageBox.Left = owner.Left + (owner.Width - _messageBox.Width) / 2;
            _messageBox.Top = owner.Top + (owner.Height - _messageBox.Height)/ 2;
            _messageBox.Owner = owner;
            
            _messageBox.ShowDialog();
            return _result;
        }
        private static void SetVisibilityOfButtons(MessageBoxButton button)
        {
            switch (button)
            {
                case MessageBoxButton.OK:
                    _messageBox.btnCancel.Visibility = Visibility.Collapsed;
                    _messageBox.btnNo.Visibility = Visibility.Collapsed;
                    _messageBox.btnYes.Visibility = Visibility.Collapsed;
                    _messageBox.btnOk.Focus();
                    break;
                case MessageBoxButton.OKCancel:
                    _messageBox.btnNo.Visibility = Visibility.Collapsed;
                    _messageBox.btnYes.Visibility = Visibility.Collapsed;
                    _messageBox.btnCancel.Focus();
                    break;
                case MessageBoxButton.YesNo:
                    _messageBox.btnOk.Visibility = Visibility.Collapsed;
                    _messageBox.btnCancel.Visibility = Visibility.Collapsed;
                    _messageBox.btnNo.Focus();
                    break;
                case MessageBoxButton.YesNoCancel:
                    _messageBox.btnOk.Visibility = Visibility.Collapsed;
                    _messageBox.btnCancel.Focus();
                    break;
            }
        }
        private static void SetImageOfMessageBox(MessageBoxExImage image)
        {
            switch (image)
            {
                case MessageBoxExImage.Warning:
                    _messageBox.SetImage("Warning.ico");
                    break;
                case MessageBoxExImage.Question:
                    _messageBox.SetImage("Question.ico");
                    break;
                case MessageBoxExImage.Information:
                    _messageBox.SetImage("Information.ico");
                    break;
                case MessageBoxExImage.Error:
                    _messageBox.SetImage("Error.ico");
                    break;
                default:
                    _messageBox.img.Visibility = Visibility.Collapsed;
                    break;
            }
        }
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (sender == btnOk)
                _result = MessageBoxResult.OK;
            else if (sender == btnYes)
                _result = MessageBoxResult.Yes;
            else if (sender == btnNo)
                _result = MessageBoxResult.No;
            else if (sender == btnCancel)
                _result = MessageBoxResult.Cancel;
            else
                _result = MessageBoxResult.None;
            _messageBox.Close();
            _messageBox = null;
        }
        private void SetImage(string imageName)
        {
            string uri = @"/Resources/" + imageName;
            var uriSource = new Uri(uri, UriKind.RelativeOrAbsolute);
            img.Source = new BitmapImage(uriSource);
        }
    }
}
