﻿// <copyright file="PTestModel.cs" company="Sirona Dental Systems GmbH">
//    Copyright (c) Sirona Dental Systems GmbH. All rights reserved.
// </copyright>

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Xml.Serialization;

namespace PTest
{
    public enum SlidaTokens
    {
        A, // Auto-select patient/image
        I, // Image info
        M, // Image mail
        S, // Select patient
        N, // New patient
        T, // Take
        U, // Update patient
        X  // X-ray order
    }
    
    public class PTestModel
    {
        #region publicModelData

        public string Name { get; set; }
        public string FirstName { get; set; }
        public string DateOfBirth { get; set; }
        public string ExtCardIndexNo { get; set; }
        public string NameNew { get; set; }
        public string FirstNameNew { get; set; }
        public string DateOfBirthNew { get; set; }
        public string ExtCardIndexNoNew { get; set; }
        public string SexNew { get; set; }
        public string PregnancyNew { get; set; }
        public string PermanentDentistNew { get; set; }
        public string Computername { get; set; }
        public string OrderDate { get; set; }
        public string OrderTime { get; set; }
        public string Sender { get; set; }
        public string Receiver { get; set; }
        public string OrderNo { get; set; }
        public string ImageNo { get; set; }
        public string ImageType { get; set; }
        public string DateOfImage { get; set; }
        public string TimeOfImage { get; set; }
        public string ResponsiblePerson { get; set; }
        public string Reason { get; set; }
        public string Diagnosis { get; set; }
        public string RadiationTime { get; set; }
        public string TubeVoltage { get; set; }
        public string TubeCurrent { get; set; }
        public string Filename { get; set; }

        #endregion
        
        [XmlIgnoreAttribute]
        private int    _generateDataCounter;
        private Random _randomGenerator;

        public PTestModel()
        {
            Name = string.Empty;
            FirstName = string.Empty;
            DateOfBirth  = string.Empty;
            ExtCardIndexNo = string.Empty;
            NameNew = string.Empty;
            FirstNameNew = string.Empty;
            DateOfBirthNew = string.Empty;
            ExtCardIndexNoNew = string.Empty;
            SexNew = string.Empty;
            PregnancyNew = string.Empty;
            PermanentDentistNew = string.Empty;
            OrderDate = string.Empty;
            OrderTime = string.Empty;
            Sender = string.Empty;
            Receiver = string.Empty;
            OrderNo = string.Empty;
            ImageNo = string.Empty;
            ImageType = string.Empty;
            DateOfImage = string.Empty;
            TimeOfImage = string.Empty;
            ResponsiblePerson = string.Empty;
            Reason = string.Empty;
            Diagnosis = string.Empty;
            RadiationTime = string.Empty;
            TubeVoltage = string.Empty;
            TubeCurrent = string.Empty;
            Filename = string.Empty;
            Computername = Environment.MachineName;
            _randomGenerator = new Random();
        }

        public void SaveData()
        {
            ((App) System.Windows.Application.Current).Save();
        }
        
        public void LoadData()
        {
            ((App) System.Windows.Application.Current).Load();
        }

        public void CreateData()
        {
            _generateDataCounter++;

            Name = GenerateStringInc( Name );
            FirstName = GenerateStringInc( FirstName );

            var patBirth = GenerateRandomDateTime( new DateTime( 1920, 1, 1 ) );

            DateOfBirth = patBirth.ToString( "dd.MM.yyyy" );
            ExtCardIndexNo = GenerateStringInc( ExtCardIndexNo );
            NameNew = GenerateStringInc( NameNew );
            FirstNameNew = GenerateStringInc( FirstNameNew );
            DateOfBirthNew = GenerateRandomDateTime( new DateTime( 1920, 1, 1 ) ).ToString( "dd.MM.yyyy" );

            ExtCardIndexNoNew = GenerateStringInc( ExtCardIndexNoNew );

            SexNew = _randomGenerator.Next(1, 3) == 1 ? "M" : "F"; // 1:1

            if (SexNew == "M") // Male
                PregnancyNew = "N"; // Not pregnant
            else
                PregnancyNew = _randomGenerator.Next( 0, 16 ) == 0 ? "P" : "N"; // 1:30

            OrderNo = GenerateIntStringInc( OrderNo );
            ImageNo = GenerateIntStringInc( ImageNo );
            ImageType = GenerateRandomTakeType();

            var minTakeDate = patBirth > new DateTime( 1990, 1, 1 ) ? patBirth : new DateTime( 1990, 1, 1 );
            var dateTimeImage = GenerateRandomDateTime( minTakeDate );

            DateOfImage = dateTimeImage.ToString( "dd.MM.yyyy" );
            TimeOfImage = dateTimeImage.ToString( "hh:mm:ss" );
            Reason = GenerateStringInc( Reason );
            Diagnosis = GenerateStringInc( Diagnosis );
        }

        public void TaskSwitch()
        {
            try
            {
                Process.Start( ((App) Application.Current).sidexisFile );
            }
            catch (Exception e)
            {
                MessageBoxEx.Show( PTest.Properties.Resources.ApplicationName,
                    PTest.Properties.Resources.ApplicationData_ErrorStart + "\n" + ((App) Application.Current).sidexisFile + "\n" + e.Message,
                    MessageBoxButton.OK, MessageBoxExImage.Error );
            }
        }
        public void ClearData()
        {
            Name = string.Empty;
            FirstName = string.Empty;
            DateOfBirth = string.Empty;
            ExtCardIndexNo = string.Empty;

            NameNew = string.Empty;
            FirstNameNew = string.Empty;
            DateOfBirthNew = string.Empty;

            ExtCardIndexNoNew = string.Empty;
            SexNew = string.Empty;
            PregnancyNew = string.Empty;

            OrderNo = string.Empty;
            ImageNo = string.Empty;
            ImageType = string.Empty;
            Reason = string.Empty;
            Diagnosis = string.Empty;

            Computername = string.Empty;
            OrderDate = string.Empty;
            OrderTime = string.Empty;
            Sender = string.Empty;
            Receiver = string.Empty;
        }

        private string GenerateStringInc( string stringToInc )
        {
            var numberPartLength = 0;

            for (var i = stringToInc.Length - 1; i >= 0; i--)
            {
                if (char.IsDigit( stringToInc[i] ) == false)
                    break;
                numberPartLength ++;
            }
            int num;
            
            if (int.TryParse( stringToInc.Substring( stringToInc.Length - numberPartLength, numberPartLength ), out num ))
            {
                num++;
                stringToInc = stringToInc.Substring( 0, stringToInc.Length - numberPartLength ) + num.ToString( "000" );
            }
            else
                stringToInc += _generateDataCounter.ToString( "000" );

            return stringToInc;
        }

        private string GenerateIntStringInc( string stringToInc )
        {
            if (stringToInc == string.Empty)
                stringToInc = "0";

            int num;

            if (int.TryParse( stringToInc, out num ))
            {
                num++;
                stringToInc = num.ToString();
            }
            return stringToInc;
        }

        private DateTime GenerateRandomDateTime( DateTime minDate )
        {
            DateTime maxDate = DateTime.Now;

            maxDate = maxDate.AddYears( -4 ); // Else patient too young :-)

            TimeSpan timeSpan = maxDate - minDate;
            TimeSpan newSpan  = new TimeSpan( 0, _randomGenerator.Next( 0, (int) timeSpan.TotalMinutes ), 0 );

            return minDate + newSpan;
        }

        private string GenerateRandomTakeType()
        {
            var type0 = " ";
            var type1 = " ";
            var type3 = " ";
            
            var nextNumber = _randomGenerator.Next( 0, 25 ); // Image type Video or X-ray, 1:24
            var type2      = nextNumber > 0 ? "X" : "V";

            switch (type2)
            {
                case "V":
                    type3 = "I";
                    type0 = _randomGenerator.Next( 1, 5 ).ToString();
                    type1 = _randomGenerator.Next( 1, 9 ).ToString();
                    break;

                case "X":

                    switch (_randomGenerator.Next(0, 4))
                    {
                        case 0: // Intra-Oral
                            type3 = "I";
                            type0 = _randomGenerator.Next(1, 5).ToString();
                            type1 = _randomGenerator.Next(1, 9).ToString();
                            break;

                        case 1: // Ceph
                            type3 = "C";
                            type0 = _randomGenerator.Next(1, 3).ToString();
                            type1 = _randomGenerator.Next(1, 3).ToString();
                            break;

                        case 2: // Pan
                            type3 = "P";

                            var programm = _randomGenerator.Next(1, 17).ToString("00");

                            type0 = programm.Substring(0, 1);
                            type1 = programm.Substring(1, 1);
                            break;

                        case 3: // 3D
                            type3 = "V";
                            type0 = " ";
                            type1 = " ";
                            break;
                    }
                    break;
            }
            return type0 + type1 + type2 + type3;
        }

        public void SendData(string textBoxMailslot, SlidaTokens tokenSelection)
        {
            var tokenData = new List<string>();

            switch (tokenSelection)
            {
                case SlidaTokens.A:
                    tokenData.Add(Name);
                    tokenData.Add(FirstName);
                    tokenData.Add(DateOfBirth);
                    tokenData.Add(ExtCardIndexNo);
                    tokenData.Add(Computername);
                    tokenData.Add(OrderDate);
                    tokenData.Add(OrderTime);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    tokenData.Add(ImageNo);
                    break;

                case SlidaTokens.I:
                    tokenData.Add(Name);
                    tokenData.Add(FirstName);
                    tokenData.Add(DateOfBirth);
                    tokenData.Add(ExtCardIndexNo);
                    tokenData.Add(ImageNo);
                    tokenData.Add(ImageType);
                    tokenData.Add(DateOfImage);
                    tokenData.Add(TimeOfImage);
                    tokenData.Add(ResponsiblePerson);
                    tokenData.Add(Reason);
                    tokenData.Add(Diagnosis);
                    tokenData.Add(RadiationTime);
                    tokenData.Add(TubeVoltage);
                    tokenData.Add(TubeCurrent);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    break;

                case SlidaTokens.M:
                    tokenData.Add(Name);
                    tokenData.Add(FirstName);
                    tokenData.Add(DateOfBirth);
                    tokenData.Add(ExtCardIndexNo);
                    tokenData.Add(ImageType);
                    tokenData.Add(DateOfImage);
                    tokenData.Add(TimeOfImage);
                    tokenData.Add(ResponsiblePerson);
                    tokenData.Add(Filename);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    tokenData.Add(ImageNo);
                    break;

                case SlidaTokens.S:
                    tokenData.Add(Name);
                    tokenData.Add(FirstName);
                    tokenData.Add(DateOfBirth);
                    tokenData.Add(ExtCardIndexNo);
                    tokenData.Add(Computername);
                    tokenData.Add(OrderDate);
                    tokenData.Add(OrderTime);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    break;

                case SlidaTokens.N:
                    tokenData.Add(NameNew);
                    tokenData.Add(FirstNameNew);
                    tokenData.Add(DateOfBirthNew);
                    tokenData.Add(ExtCardIndexNoNew);
                    tokenData.Add(SexNew);
                    tokenData.Add(PermanentDentistNew);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    break;

                case SlidaTokens.T:
                    tokenData.Add(OrderNo);
                    tokenData.Add(Name);
                    tokenData.Add(FirstName);
                    tokenData.Add(DateOfBirth);
                    tokenData.Add(ExtCardIndexNo);
                    tokenData.Add(ImageNo);
                    tokenData.Add(SexNew);
                    tokenData.Add(PregnancyNew);
                    tokenData.Add(ImageType);
                    tokenData.Add(DateOfImage);
                    tokenData.Add(TimeOfImage);
                    tokenData.Add(ResponsiblePerson);
                    tokenData.Add(Reason);
                    tokenData.Add(Diagnosis);
                    tokenData.Add(RadiationTime);
                    tokenData.Add(TubeVoltage);
                    tokenData.Add(TubeCurrent);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    break;

                case SlidaTokens.U:
                    tokenData.Add(Name);
                    tokenData.Add(FirstName);
                    tokenData.Add(DateOfBirth);
                    tokenData.Add(ExtCardIndexNo);
                    tokenData.Add(NameNew);
                    tokenData.Add(FirstNameNew);
                    tokenData.Add(DateOfBirthNew);
                    tokenData.Add(ExtCardIndexNoNew);
                    tokenData.Add(SexNew);
                    tokenData.Add(PermanentDentistNew);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    break;

                case SlidaTokens.X:
                    tokenData.Add(OrderNo);
                    tokenData.Add(Name);
                    tokenData.Add(FirstName);
                    tokenData.Add(DateOfBirth);
                    tokenData.Add(ExtCardIndexNo);
                    tokenData.Add(SexNew);
                    tokenData.Add(PregnancyNew);
                    tokenData.Add(ImageType);
                    tokenData.Add(Reason);
                    tokenData.Add(Computername);
                    tokenData.Add(OrderDate);
                    tokenData.Add(OrderTime);
                    tokenData.Add(Sender);
                    tokenData.Add(Receiver);
                    break;

                default:
                    throw new ArgumentOutOfRangeException("tokenSelection", tokenSelection, null);
            }
            var token = Slida.GenerateToken(tokenSelection.ToString()[0], tokenData);

            Slida.WriteToSlida(textBoxMailslot, token);
        }
    }
}
